from argschema import ArgSchema
from argschema.schemas import DefaultSchema
from argschema.fields import Nested, String, Float, List, Int

from .drifting_gratings import DriftingGratings
from .static_gratings import StaticGratings
from .natural_scenes import NaturalScenes
from .dot_motion import DotMotion
from .flashes import Flashes
from .receptive_field_mapping import ReceptiveFieldMapping


class DriftingGratings(DefaultSchema):
    stimulus_key = List(String, default=DriftingGratings.known_stimulus_keys(), help='Key for the drifting gratings stimulus')
    trial_duration = Float(default=2.0, help='typical length of a epoch for given stimulus in seconds')
    psth_resolution = Float(default=0.001, help='resultion (seconds) for generating PSTH')



class StaticGratings(DefaultSchema):
    stimulus_key = List(String, default=StaticGratings.known_stimulus_keys(), help='Key for the static gratings stimulus')
    trial_duration = Float(default=0.25, help='typical length of a epoch for given stimulus in seconds')
    psth_resolution = Float(default=0.001, help='resultion (seconds) for generating PSTH')


class NaturalScenes(DefaultSchema):
    stimulus_key = List(String, default=NaturalScenes.known_stimulus_keys(), help='Key for the natural scenes stimulus')
    trial_duration = Float(default=0.25, help='typical length of a epoch for given stimulus in seconds')
    psth_resolution = Float(default=0.001, help='resultion (seconds) for generating PSTH')


#class NaturalMovies(DefaultSchema):
#    stimulus_key = String(help='Key for the natural movies stimulus')
#    trial_duration = Float(default=0.25, help='typical length of a epoch for given stimulus in seconds')


class DotMotion(DefaultSchema):
    stimulus_key = List(String, default=DotMotion.known_stimulus_keys(), help='Key for the dot motion stimulus')
    trial_duration = Float(default=1.0, help='typical length of a epoch for given stimulus in seconds')
    psth_resolution = Float(default=0.001, help='resultion (seconds) for generating PSTH')


#class ContrastTuning(DefaultSchema):
#    stimulus_key = String(help='Key for the contrast tuning stimulus')
#    trial_duration = Float(default=0.25, help='typical length of a epoch for given stimulus in seconds')


class Flashes(DefaultSchema):
    stimulus_key = List(String, default=Flashes.known_stimulus_keys(), help='Key for the flash stimulus')
    trial_duration = Float(default=0.25, help='typical length of a epoch for given stimulus in seconds')
    psth_resolution = Float(default=0.001, help='resultion (seconds) for generating PSTH')


class ReceptiveFieldMapping(DefaultSchema):
    stimulus_key = List(String, default=ReceptiveFieldMapping.known_stimulus_keys(), help='Key for the receptive field mapping stimulus')
    trial_duration = Float(default=0.25, help='typical length of a epoch for given stimulus in seconds')
    minimum_spike_count = Int(default=10, help='Minimum number of spikes for computing receptive field parameters')
    mask_threshold = Float(default=1.0, help='Threshold (as fraction of peak) for computing receptive field mask')
    stimulus_step_size = Float(default=10.0, help='Distance between stimulus locations in degrees')


class InputParameters(ArgSchema):
    drifting_gratings = Nested(DriftingGratings)
    static_gratings = Nested(StaticGratings)
    natural_scenes = Nested(NaturalScenes)
    # natural_movies = Nested(NaturalMovies)
    dot_motion = Nested(DotMotion)
    # contrast_tuning = Nested(ContrastTuning)
    flashes = Nested(Flashes)
    receptive_field_mapping = Nested(ReceptiveFieldMapping)

    input_session_nwb = String(required=True, help='Ecephys spiking nwb file for session')
    output_file = String(required=True, help='Location for saving output file')


class OutputSchema(DefaultSchema):
    input_parameters = Nested(InputParameters,
                              description=("Input parameters the module was run with"),
                              required=True)


class OutputParameters(OutputSchema):
    execution_time = Float()
