import time
t0 = time.time()
import os

from matplotlib.patches import Ellipse
import matplotlib.pyplot as plt
from moviepy.video.io.bindings import mplfig_to_npimage
from moviepy.editor import *
import numpy as np
import collections
import pandas as pd
import sys
import re
import argparse
import logging
from ellipses import LSqEllipse
from google.cloud import storage


ch = logging.StreamHandler()
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
ch.setFormatter(formatter)
logger = logging.getLogger('dlc-ellipse-fitting')
logger.setLevel(logging.INFO)
logger.addHandler(ch)
logger.propagate = False

parser = argparse.ArgumentParser()
parser.add_argument("--video_input_file", type=str, required=True, help="path to video file, mp4 or avi")
args = parser.parse_args()

video_file_path = args.video_input_file
ellipse_bucket_data_blobname = '{}.h5'.format(os.path.splitext(video_file_path)[0])
source_ellipse_data_file = '/workdir/{}'.format(ellipse_bucket_data_blobname)
source_ellipse_data_file = ellipse_bucket_data_blobname

client = storage.Client()
fit_src_bucket = client.get_bucket('dlc-ellipse-fitting')
blob = fit_src_bucket.get_blob(ellipse_bucket_data_blobname)
blob.download_to_filename(source_ellipse_data_file)

movie_src_bucket = client.get_bucket('brain-observatory-eye-videos')
blob = movie_src_bucket.get_blob(video_file_path)
blob.download_to_filename(video_file_path)

ellipse_output_blob_name = "{}_ellipse_output_video_file.mp4".format(os.path.splitext(video_file_path)[0])
ellipse_output_video_file = "/workdir/{}".format(ellipse_output_blob_name)
ellipse_output_video_file = ellipse_output_blob_name

cr = pd.read_hdf(source_ellipse_data_file, key='cr')
eye = pd.read_hdf(source_ellipse_data_file, key='eye')
pupil = pd.read_hdf(source_ellipse_data_file, key='pupil')

def make_frame(t):      

    fi = np.int(np.round(t*fps))

    ax.clear()
    ax.imshow(clip.get_frame(t))
    #that is the pupi; ellipse in red 
    try:
        ellipse = Ellipse((cr.loc[fi]['center_x'], cr.loc[fi]['center_y']), 2*cr.loc[fi]['width'], 2*cr.loc[fi]['height'], np.rad2deg(cr.loc[fi]['phi']), alpha=0.8, ec='r', fc=None, lw=2,  fill=False)
        ax.add_patch(ellipse)
    except Exception as e:
        print(e)
        #that is the eye ellipse in green
    try:
        ellipse = Ellipse((eye.loc[fi]['center_x'], eye.loc[fi]['center_y']), 2*eye.loc[fi]['width'], 2*eye.loc[fi]['height'], np.rad2deg(eye.loc[fi]['phi']), alpha=0.8, ec='g', fc=None, lw=2,  fill=False)
        ax.add_patch(ellipse)
    except Exception as e:
        print(e)

        #Corneal reflection in blue
    try:
        ellipse = Ellipse((pupil.loc[fi]['center_x'], pupil.loc[fi]['center_y']), 2*pupil.loc[fi]['width'], 2*pupil.loc[fi]['height'], np.rad2deg(pupil.loc[fi]['phi']), alpha=0.8, ec='b', fc=None, lw=2,  fill=False)
        ax.add_patch(ellipse)
        ax.set_axis_off()
    except Exception as e:
        print(e)

    return mplfig_to_npimage(fig)

initialization_time = time.time() - t0
ellipse_video_t0 = time.time()

fps = 30.0
clip = VideoFileClip(video_file_path)
fig, ax = plt.subplots()
fig.set_size_inches([6.4, 4.8], forward=True)
ax.set_xlim(0, clip.size[0])
ax.set_ylim(0, clip.size[1])
fig.subplots_adjust(left=0, bottom=0, right=1, top=1, wspace=None, hspace=None)

animation = VideoClip(make_frame, duration=clip.duration).resize(newsize=clip.size)

animation.write_videofile(ellipse_output_video_file, fps=fps)
tgt_bucket = client.get_bucket('dlc-ellipse-videos')
blob2 = tgt_bucket.blob(ellipse_output_blob_name)
blob2.upload_from_filename(filename=ellipse_output_video_file)

ellipse_video_time = time.time() - ellipse_video_t0

logger.info('Initialization Time: {}'.format(initialization_time))
# logger.info('Ellipse Video Time: {}'.format(ellipse_video_time))
logger.info('Total Walltime: {}'.format(time.time()-t0))



