import pytest
import sys
from mock import patch, mock_open, Mock, MagicMock
from allensdk.model.biophysical.utils import Utils
from allensdk.model.biophys_sim.config import Config
import os
import mock
import shutil
try:
    import __builtin__ as builtins
except:
    import builtins
from allensdk.model.biophysical import runner
from allensdk.internal.model.biophysical.run_optimize \
    import RunOptimize
from allensdk.internal.api.queries.optimize_config_reader \
    import OptimizeConfigReader
from allensdk.model.biophys_sim.neuron.hoc_utils import HocUtils

real_import = __import__

#import allensdk.eclipse_debug


MANIFEST_JSON = '''
{
  "biophys": [
    {
      "model_file": [
        "/local1/tmp/manifest_sdk.json"
      ]
    }
  ], 
  "runs": [
    {
      "specimen_id": 318733871, 
      "sweeps": [
        4, 
        5, 
        6, 
        7, 
        8, 
        9, 
        10, 
        11, 
        12, 
        13, 
        14, 
        15, 
        16, 
        17, 
        18, 
        19, 
        20, 
        21, 
        22, 
        23, 
        24, 
        25, 
        26, 
        27, 
        30, 
        31, 
        32, 
        33, 
        34, 
        35, 
        36, 
        37, 
        38, 
        39, 
        40, 
        41, 
        42, 
        43, 
        44, 
        45, 
        47, 
        48, 
        49, 
        50, 
        51, 
        52, 
        53, 
        55, 
        56, 
        57, 
        59, 
        60, 
        61, 
        62, 
        63, 
        64, 
        65, 
        66, 
        67, 
        68, 
        69, 
        70, 
        71, 
        72, 
        73, 
        74, 
        75, 
        76, 
        77, 
        78, 
        79, 
        80, 
        81, 
        82, 
        83, 
        84, 
        85, 
        86, 
        87, 
        88, 
        89, 
        90, 
        91, 
        92, 
        93, 
        94, 
        95, 
        96, 
        97, 
        98, 
        99, 
        100, 
        101, 
        102, 
        103, 
        106
      ]
    }
  ], 
  "neuron": [
    {
      "hoc": [
        "stdgui.hoc", 
        "import3d.hoc", 
        "cell.hoc"
      ]
    }
  ], 
  "manifest": [
    {
      "type": "dir", 
      "spec": "/local1/tmp", 
      "key": "BASEDIR"
    }, 
    {
      "type": "dir", 
      "spec": "/projects/mousecelltypes/vol1/prod192/neuronal_model_329322394/work", 
      "key": "WORKDIR"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod155/specimen_318733871/Nr5a1-Cre_Ai14-169248.04.02.01_491253825_m.swc", 
      "key": "MORPHOLOGY"
    }, 
    {
      "type": "dir", 
      "spec": "modfiles", 
      "key": "MODFILE_DIR"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod555/project_in vitro Single Cell Characterization_T301/Kv2like.mod", 
      "key": "MOD_FILE_Kv2like", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod251/project_in vitro Single Cell Characterization_T301/NaV.mod", 
      "key": "MOD_FILE_NaV", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Ca_HVA.mod", 
      "key": "MOD_FILE_Ca_HVA", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Ca_LVA.mod", 
      "key": "MOD_FILE_Ca_LVA", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/CaDynamics.mod", 
      "key": "MOD_FILE_CaDynamics", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Ih.mod", 
      "key": "MOD_FILE_Ih", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Im.mod", 
      "key": "MOD_FILE_Im", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Im_v2.mod", 
      "key": "MOD_FILE_Im_v2", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/K_P.mod", 
      "key": "MOD_FILE_K_P", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/K_T.mod", 
      "key": "MOD_FILE_K_T", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Kd.mod", 
      "key": "MOD_FILE_Kd", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Kv3_1.mod", 
      "key": "MOD_FILE_Kv3_1", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Nap.mod", 
      "key": "MOD_FILE_Nap", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/NaTa.mod", 
      "key": "MOD_FILE_NaTa", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/NaTs.mod", 
      "key": "MOD_FILE_NaTs", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/SK.mod", 
      "key": "MOD_FILE_SK", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod555/project_in vitro Single Cell Characterization_T301/Kv2like.mod", 
      "key": "MOD_FILE_Kv2like", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod251/project_in vitro Single Cell Characterization_T301/NaV.mod", 
      "key": "MOD_FILE_NaV", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Ca_HVA.mod", 
      "key": "MOD_FILE_Ca_HVA", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Ca_LVA.mod", 
      "key": "MOD_FILE_Ca_LVA", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/CaDynamics.mod", 
      "key": "MOD_FILE_CaDynamics", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Ih.mod", 
      "key": "MOD_FILE_Ih", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Im.mod", 
      "key": "MOD_FILE_Im", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Im_v2.mod", 
      "key": "MOD_FILE_Im_v2", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/K_P.mod", 
      "key": "MOD_FILE_K_P", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/K_T.mod", 
      "key": "MOD_FILE_K_T", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Kd.mod", 
      "key": "MOD_FILE_Kd", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Kv3_1.mod", 
      "key": "MOD_FILE_Kv3_1", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Nap.mod", 
      "key": "MOD_FILE_Nap", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/NaTa.mod", 
      "key": "MOD_FILE_NaTa", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/NaTs.mod", 
      "key": "MOD_FILE_NaTs", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/SK.mod", 
      "key": "MOD_FILE_SK", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod555/project_in vitro Single Cell Characterization_T301/Kv2like.mod", 
      "key": "MOD_FILE_Kv2like", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod251/project_in vitro Single Cell Characterization_T301/NaV.mod", 
      "key": "MOD_FILE_NaV", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Ca_HVA.mod", 
      "key": "MOD_FILE_Ca_HVA", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Ca_LVA.mod", 
      "key": "MOD_FILE_Ca_LVA", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/CaDynamics.mod", 
      "key": "MOD_FILE_CaDynamics", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Ih.mod", 
      "key": "MOD_FILE_Ih", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Im.mod", 
      "key": "MOD_FILE_Im", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Im_v2.mod", 
      "key": "MOD_FILE_Im_v2", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/K_P.mod", 
      "key": "MOD_FILE_K_P", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/K_T.mod", 
      "key": "MOD_FILE_K_T", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Kd.mod", 
      "key": "MOD_FILE_Kd", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Kv3_1.mod", 
      "key": "MOD_FILE_Kv3_1", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/Nap.mod", 
      "key": "MOD_FILE_Nap", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/NaTa.mod", 
      "key": "MOD_FILE_NaTa", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/NaTs.mod", 
      "key": "MOD_FILE_NaTs", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/SK.mod", 
      "key": "MOD_FILE_SK", 
      "format": "MODFILE"
    }, 
    {
      "type": "file", 
      "spec": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/318733869.nwb", 
      "key": "stimulus_path", 
      "format": "NWB"
    }, 
    {
      "type": "file", 
      "spec": "/local1/tmp/manifest_sdk.json", 
      "key": "manifest"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "318733869.nwb", 
      "key": "output", 
      "format": "NWB"
    }, 
    {
      "type": "file", 
      "spec": "/local1/tmp/lims_message_optimize.json", 
      "key": "neuronal_model_data"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "upbase.dat", 
      "key": "upfile"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "downbase.dat", 
      "key": "downfile"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "passive_fit_data.json", 
      "key": "passive_fit_data"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "stage_1_jobs.json", 
      "key": "stage_1_jobs"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "fit_1_data.json", 
      "key": "fit_1_file"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "fit_2_data.json", 
      "key": "fit_2_file"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "fit_3_data.json", 
      "key": "fit_3_file"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "%s", 
      "key": "fit_type_path"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "target.json", 
      "key": "target_path"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "%s/config.json", 
      "key": "fit_config_json"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "%s/s%d/final_hof_fit.txt", 
      "key": "final_hof_fit"
    }, 
    {
      "parent_key": "WORKDIR", 
      "type": "file", 
      "spec": "%s/s%d/final_hof.txt", 
      "key": "final_hof"
    }, 
    {
      "type": "file", 
      "spec": "fit_%s_%s.json", 
      "key": "output_fit_file"
    }
  ]
}
'''

LIMS_MESSAGE = '''
{
  "created_at": "2015-02-13T16:52:57-08:00", 
  "id": 329322394, 
  "name": "Biophysical - perisomatic_Nr5a1-Cre;Ai14-169248.04.02.01", 
  "neuronal_model_template": {
    "created_at": "2015-02-13T11:51:50-08:00", 
    "id": 329230710, 
    "name": "Biophysical - perisomatic", 
    "neuronal_model_template_type": {
      "created_at": "2015-02-12T19:46:16-08:00", 
      "description": "Biophysical Neuronal Model Template", 
      "id": 328959041, 
      "name": "BIOPHYS", 
      "updated_at": "2016-02-12T15:30:27-08:00"
    }, 
    "neuronal_model_template_type_id": 328959041, 
    "updated_at": "2015-02-13T11:51:50-08:00", 
    "well_known_files": [
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "text/plain; charset=us-ascii", 
        "created_at": "2015-11-13T13:47:51-08:00", 
        "file_source_id": null, 
        "filename": "Kv2like.mod", 
        "id": 491113425, 
        "published_at": "2016-03-03", 
        "size": 1433, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod555/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-11-13T13:47:51-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "audio/x-mod", 
        "created_at": "2015-03-16T11:07:34-07:00", 
        "file_source_id": null, 
        "filename": "NaV.mod", 
        "id": 464138096, 
        "published_at": "2015-05-14", 
        "size": 4061, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod251/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-03-16T11:08:04-07:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:20-08:00", 
        "file_source_id": null, 
        "filename": "Ca_HVA.mod", 
        "id": 395337003, 
        "published_at": "2015-05-14", 
        "size": 1211, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:05-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:30-08:00", 
        "file_source_id": null, 
        "filename": "Ca_LVA.mod", 
        "id": 395337007, 
        "published_at": "2015-05-14", 
        "size": 1069, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:13-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:40-08:00", 
        "file_source_id": null, 
        "filename": "CaDynamics.mod", 
        "id": 395337011, 
        "published_at": "2015-05-14", 
        "size": 704, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:20-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:48-08:00", 
        "file_source_id": null, 
        "filename": "Ih.mod", 
        "id": 395337015, 
        "published_at": "2015-05-14", 
        "size": 1005, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:27-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:57-08:00", 
        "file_source_id": null, 
        "filename": "Im.mod", 
        "id": 395337019, 
        "published_at": "2015-05-14", 
        "size": 859, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:34-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:46:21-08:00", 
        "file_source_id": null, 
        "filename": "Im_v2.mod", 
        "id": 395337042, 
        "published_at": "2015-05-14", 
        "size": 761, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:40-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:46:29-08:00", 
        "file_source_id": null, 
        "filename": "K_P.mod", 
        "id": 395337046, 
        "published_at": "2015-05-14", 
        "size": 1161, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:47-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:46:42-08:00", 
        "file_source_id": null, 
        "filename": "K_T.mod", 
        "id": 395337050, 
        "published_at": "2015-05-14", 
        "size": 1031, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:53-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:47:02-08:00", 
        "file_source_id": null, 
        "filename": "Kd.mod", 
        "id": 395337054, 
        "published_at": "2015-05-14", 
        "size": 705, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:59-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:47:20-08:00", 
        "file_source_id": null, 
        "filename": "Kv3_1.mod", 
        "id": 395337062, 
        "published_at": "2015-05-14", 
        "size": 631, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:13-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:47:29-08:00", 
        "file_source_id": null, 
        "filename": "Nap.mod", 
        "id": 395337066, 
        "published_at": "2015-05-14", 
        "size": 1170, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:19-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:47:38-08:00", 
        "file_source_id": null, 
        "filename": "NaTa.mod", 
        "id": 395337070, 
        "published_at": "2015-05-14", 
        "size": 1413, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:29-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:48:22-08:00", 
        "file_source_id": null, 
        "filename": "NaTs.mod", 
        "id": 395337225, 
        "published_at": "2015-05-14", 
        "size": 1267, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:36-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:49:02-08:00", 
        "file_source_id": null, 
        "filename": "SK.mod", 
        "id": 395337293, 
        "published_at": "2015-05-14", 
        "size": 942, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:48-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "text/plain; charset=us-ascii", 
        "created_at": "2015-11-13T13:47:51-08:00", 
        "file_source_id": null, 
        "filename": "Kv2like.mod", 
        "id": 491113425, 
        "published_at": "2016-03-03", 
        "size": 1433, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod555/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-11-13T13:47:51-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "audio/x-mod", 
        "created_at": "2015-03-16T11:07:34-07:00", 
        "file_source_id": null, 
        "filename": "NaV.mod", 
        "id": 464138096, 
        "published_at": "2015-05-14", 
        "size": 4061, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod251/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-03-16T11:08:04-07:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:20-08:00", 
        "file_source_id": null, 
        "filename": "Ca_HVA.mod", 
        "id": 395337003, 
        "published_at": "2015-05-14", 
        "size": 1211, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:05-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:30-08:00", 
        "file_source_id": null, 
        "filename": "Ca_LVA.mod", 
        "id": 395337007, 
        "published_at": "2015-05-14", 
        "size": 1069, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:13-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:40-08:00", 
        "file_source_id": null, 
        "filename": "CaDynamics.mod", 
        "id": 395337011, 
        "published_at": "2015-05-14", 
        "size": 704, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:20-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:48-08:00", 
        "file_source_id": null, 
        "filename": "Ih.mod", 
        "id": 395337015, 
        "published_at": "2015-05-14", 
        "size": 1005, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:27-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:57-08:00", 
        "file_source_id": null, 
        "filename": "Im.mod", 
        "id": 395337019, 
        "published_at": "2015-05-14", 
        "size": 859, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:34-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:46:21-08:00", 
        "file_source_id": null, 
        "filename": "Im_v2.mod", 
        "id": 395337042, 
        "published_at": "2015-05-14", 
        "size": 761, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:40-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:46:29-08:00", 
        "file_source_id": null, 
        "filename": "K_P.mod", 
        "id": 395337046, 
        "published_at": "2015-05-14", 
        "size": 1161, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:47-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:46:42-08:00", 
        "file_source_id": null, 
        "filename": "K_T.mod", 
        "id": 395337050, 
        "published_at": "2015-05-14", 
        "size": 1031, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:53-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:47:02-08:00", 
        "file_source_id": null, 
        "filename": "Kd.mod", 
        "id": 395337054, 
        "published_at": "2015-05-14", 
        "size": 705, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:59-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:47:20-08:00", 
        "file_source_id": null, 
        "filename": "Kv3_1.mod", 
        "id": 395337062, 
        "published_at": "2015-05-14", 
        "size": 631, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:13-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:47:29-08:00", 
        "file_source_id": null, 
        "filename": "Nap.mod", 
        "id": 395337066, 
        "published_at": "2015-05-14", 
        "size": 1170, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:19-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:47:38-08:00", 
        "file_source_id": null, 
        "filename": "NaTa.mod", 
        "id": 395337070, 
        "published_at": "2015-05-14", 
        "size": 1413, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:29-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:48:22-08:00", 
        "file_source_id": null, 
        "filename": "NaTs.mod", 
        "id": 395337225, 
        "published_at": "2015-05-14", 
        "size": 1267, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:36-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:49:02-08:00", 
        "file_source_id": null, 
        "filename": "SK.mod", 
        "id": 395337293, 
        "published_at": "2015-05-14", 
        "size": 942, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:48-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "text/plain; charset=us-ascii", 
        "created_at": "2015-11-13T13:47:51-08:00", 
        "file_source_id": null, 
        "filename": "Kv2like.mod", 
        "id": 491113425, 
        "published_at": "2016-03-03", 
        "size": 1433, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod555/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-11-13T13:47:51-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "audio/x-mod", 
        "created_at": "2015-03-16T11:07:34-07:00", 
        "file_source_id": null, 
        "filename": "NaV.mod", 
        "id": 464138096, 
        "published_at": "2015-05-14", 
        "size": 4061, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod251/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-03-16T11:08:04-07:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:20-08:00", 
        "file_source_id": null, 
        "filename": "Ca_HVA.mod", 
        "id": 395337003, 
        "published_at": "2015-05-14", 
        "size": 1211, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:05-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:30-08:00", 
        "file_source_id": null, 
        "filename": "Ca_LVA.mod", 
        "id": 395337007, 
        "published_at": "2015-05-14", 
        "size": 1069, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:13-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:40-08:00", 
        "file_source_id": null, 
        "filename": "CaDynamics.mod", 
        "id": 395337011, 
        "published_at": "2015-05-14", 
        "size": 704, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:20-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:48-08:00", 
        "file_source_id": null, 
        "filename": "Ih.mod", 
        "id": 395337015, 
        "published_at": "2015-05-14", 
        "size": 1005, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:27-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:42:57-08:00", 
        "file_source_id": null, 
        "filename": "Im.mod", 
        "id": 395337019, 
        "published_at": "2015-05-14", 
        "size": 859, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:34-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:46:21-08:00", 
        "file_source_id": null, 
        "filename": "Im_v2.mod", 
        "id": 395337042, 
        "published_at": "2015-05-14", 
        "size": 761, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:40-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:46:29-08:00", 
        "file_source_id": null, 
        "filename": "K_P.mod", 
        "id": 395337046, 
        "published_at": "2015-05-14", 
        "size": 1161, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:47-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:46:42-08:00", 
        "file_source_id": null, 
        "filename": "K_T.mod", 
        "id": 395337050, 
        "published_at": "2015-05-14", 
        "size": 1031, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:53-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:47:02-08:00", 
        "file_source_id": null, 
        "filename": "Kd.mod", 
        "id": 395337054, 
        "published_at": "2015-05-14", 
        "size": 705, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:50:59-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:47:20-08:00", 
        "file_source_id": null, 
        "filename": "Kv3_1.mod", 
        "id": 395337062, 
        "published_at": "2015-05-14", 
        "size": 631, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:13-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:47:29-08:00", 
        "file_source_id": null, 
        "filename": "Nap.mod", 
        "id": 395337066, 
        "published_at": "2015-05-14", 
        "size": 1170, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:19-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:47:38-08:00", 
        "file_source_id": null, 
        "filename": "NaTa.mod", 
        "id": 395337070, 
        "published_at": "2015-05-14", 
        "size": 1413, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:29-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:48:22-08:00", 
        "file_source_id": null, 
        "filename": "NaTs.mod", 
        "id": 395337225, 
        "published_at": "2015-05-14", 
        "size": 1267, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:36-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }, 
      {
        "ar_association_key_name": "329230710", 
        "attachable_id": 305094322, 
        "attachable_type": "Project", 
        "content_type": "video/mpeg", 
        "created_at": "2015-02-27T11:49:02-08:00", 
        "file_source_id": null, 
        "filename": "SK.mod", 
        "id": 395337293, 
        "published_at": "2015-05-14", 
        "size": 942, 
        "storage_directory": "/projects/mousecelltypes/vol1/prod210/project_in vitro Single Cell Characterization_T301/", 
        "updated_at": "2015-02-27T11:51:48-08:00", 
        "well_known_file_type": {
          "created_at": "2013-11-05T16:59:04-08:00", 
          "id": 292178729, 
          "name": "BiophysicalModelDescription", 
          "updated_at": "2013-11-05T16:59:04-08:00"
        }, 
        "well_known_file_type_id": 292178729, 
        "workflow_state": null
      }
    ]
  }, 
  "neuronal_model_template_id": 329230710, 
  "specimen": {
    "alignment3d_id": 473077597, 
    "barcode": "0318733871", 
    "biophysical_model_state": "review_required", 
    "carousel_well_name": null, 
    "cell_depth": null, 
    "cell_prep_id": null, 
    "cell_reporter_id": 491913822, 
    "created_at": "2015-01-07T12:12:35-08:00", 
    "created_by": null, 
    "data": null, 
    "donor_id": 318293721, 
    "ephys_cell_plan_id": 308388019, 
    "ephys_neural_tissue_plan_id": null, 
    "ephys_roi_result": {
      "blowout_mv": 3.63650708459318, 
      "created_at": "2015-01-07T12:12:34-08:00", 
      "electrode_0_pa": -2.87437500016974, 
      "ephys_qc_criteria": {
        "access_resistance_mohm_max": 20.0, 
        "access_resistance_mohm_min": 1.0, 
        "blowout_mv_max": 10.0, 
        "blowout_mv_min": -10.0, 
        "created_at": "2015-01-29T13:51:29-08:00", 
        "electrode_0_pa_max": 200.0, 
        "electrode_0_pa_min": -200.0, 
        "id": 324256702, 
        "input_vs_access_resistance_min": 0.15, 
        "leak_pa_max": 100.0, 
        "leak_pa_min": -100.0, 
        "name": "Ephys QC Criteria v1.1", 
        "post_noise_rms_mv_max": 0.07, 
        "pre_noise_rms_mv_max": 0.07, 
        "seal_gohm_min": 1.0, 
        "slow_noise_rms_mv_max": 0.5, 
        "updated_at": "2015-01-29T13:51:29-08:00", 
        "vm_delta_mv_max": 1.0
      }, 
      "ephys_qc_criteria_id": 324256702, 
      "ephys_specimen_roi_plan_id": 318696412, 
      "failed_bad_rs": false, 
      "failed_clogged_pipette": false, 
      "failed_electrode_0": false, 
      "failed_no_seal": null, 
      "failed_other": null, 
      "id": 318733869, 
      "initial_access_resistance_mohm": 12.415662, 
      "input_access_resistance_ratio": 0.0638893902110391, 
      "input_resistance_mohm": 194.330576, 
      "notes": null, 
      "published_at": "2015-05-13T17:00:00-07:00", 
      "recording_date": "2015-01-07T03:24:48-08:00", 
      "rig_name": null, 
      "seal_gohm": 2.35573504, 
      "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
      "updated_at": "2015-07-08T05:57:54-07:00", 
      "well_known_files": [
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "application/octet-stream; charset=binary", 
          "created_at": "2015-01-07T12:14:58-08:00", 
          "file_source_id": null, 
          "filename": "Nr5a1-Cre;Ai14(IVSCC)-169248.04.02.pxp", 
          "id": 318736730, 
          "published_at": null, 
          "size": 2641238729, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2015-01-07T12:14:58-08:00", 
          "well_known_file_type": {
            "created_at": "2014-07-10T19:36:42-07:00", 
            "id": 305301981, 
            "name": "IGOR Output", 
            "updated_at": "2014-07-10T19:36:42-07:00"
          }, 
          "well_known_file_type_id": 305301981, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "image/tiff; charset=binary", 
          "created_at": "2015-01-07T12:14:58-08:00", 
          "file_source_id": null, 
          "filename": "Nr5a1-Cre;Ai14(IVSCC)-169248.04.02.40x.tif", 
          "id": 318736732, 
          "published_at": null, 
          "size": 2895604, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2015-01-07T12:14:58-08:00", 
          "well_known_file_type": {
            "created_at": "2014-07-10T19:36:43-07:00", 
            "id": 305301983, 
            "name": "Cell Image", 
            "updated_at": "2014-07-10T19:36:43-07:00"
          }, 
          "well_known_file_type_id": 305301983, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "image/tiff; charset=binary", 
          "created_at": "2015-01-07T12:14:58-08:00", 
          "file_source_id": null, 
          "filename": "Nr5a1-Cre;Ai14(IVSCC)-169248.04.02.40x_patched_bf.tif", 
          "id": 318736734, 
          "published_at": null, 
          "size": 2895604, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2015-01-07T12:14:58-08:00", 
          "well_known_file_type": {
            "created_at": "2014-11-11T19:23:14-08:00", 
            "id": 310980879, 
            "name": "Cell Image: BreakIn Contrast", 
            "updated_at": "2014-11-11T19:23:14-08:00"
          }, 
          "well_known_file_type_id": 310980879, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "image/tiff; charset=binary", 
          "created_at": "2015-01-07T12:14:58-08:00", 
          "file_source_id": null, 
          "filename": "Nr5a1-Cre;Ai14(IVSCC)-169248.04.02.40x_patched_epi.tif", 
          "id": 318736736, 
          "published_at": null, 
          "size": 2895604, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2015-01-07T12:14:58-08:00", 
          "well_known_file_type": {
            "created_at": "2014-11-11T19:23:15-08:00", 
            "id": 310980881, 
            "name": "Cell Image: BreakIn 565nm", 
            "updated_at": "2014-11-11T19:23:15-08:00"
          }, 
          "well_known_file_type_id": 310980881, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "image/tiff; charset=binary", 
          "created_at": "2015-01-07T12:14:58-08:00", 
          "file_source_id": null, 
          "filename": "Nr5a1-Cre;Ai14(IVSCC)-169248.04.02.4x.tif", 
          "id": 318736738, 
          "published_at": null, 
          "size": 2895604, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2015-01-07T12:14:58-08:00", 
          "well_known_file_type": {
            "created_at": "2014-08-14T14:53:07-07:00", 
            "id": 306905520, 
            "name": "Low Magnification Image", 
            "updated_at": "2014-08-14T14:53:07-07:00"
          }, 
          "well_known_file_type_id": 306905520, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "application/x-hdf; charset=binary", 
          "created_at": "2015-01-07T12:14:58-08:00", 
          "file_source_id": null, 
          "filename": "Nr5a1-Cre;Ai14(IVSCC)-169248.04.02.h5", 
          "id": 318736740, 
          "published_at": null, 
          "size": 2643186456, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2015-01-07T12:14:58-08:00", 
          "well_known_file_type": {
            "created_at": "2014-08-14T14:53:09-07:00", 
            "id": 306905526, 
            "name": "HDF5", 
            "updated_at": "2014-08-14T14:53:09-07:00"
          }, 
          "well_known_file_type_id": 306905526, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "application/json", 
          "created_at": "2015-07-08T06:55:38-07:00", 
          "file_source_id": null, 
          "filename": "318733869_ephys_features.json", 
          "id": 480630908, 
          "published_at": null, 
          "size": null, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2015-07-08T06:55:38-07:00", 
          "well_known_file_type_id": null, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "image/png; charset=binary", 
          "created_at": "2015-10-02T11:03:09-07:00", 
          "file_source_id": null, 
          "filename": "morphology_summary.png", 
          "id": 487660096, 
          "published_at": "2015-05-14", 
          "size": 988, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2015-11-30T20:38:38-08:00", 
          "well_known_file_type": {
            "created_at": "2015-07-09T13:57:28-07:00", 
            "id": 480715721, 
            "name": "MorphologyThumbnail", 
            "updated_at": "2015-07-09T13:57:28-07:00"
          }, 
          "well_known_file_type_id": 480715721, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "image/png; charset=binary", 
          "created_at": "2015-10-01T14:46:12-07:00", 
          "file_source_id": null, 
          "filename": "ephys_summary.png", 
          "id": 487611469, 
          "published_at": "2015-05-14", 
          "size": 9227, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2016-01-29T02:02:59-08:00", 
          "well_known_file_type": {
            "created_at": "2015-07-09T13:58:28-07:00", 
            "id": 480715749, 
            "name": "EphysSummaryThumbnail", 
            "updated_at": "2015-07-09T13:58:28-07:00"
          }, 
          "well_known_file_type_id": 480715749, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "image/png; charset=binary", 
          "created_at": "2015-11-30T21:30:02-08:00", 
          "file_source_id": null, 
          "filename": "ephys_inst_threshold.png", 
          "id": 491381583, 
          "published_at": "2015-05-14", 
          "size": 1477, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2016-01-29T02:02:59-08:00", 
          "well_known_file_type": {
            "created_at": "2015-10-14T10:14:10-07:00", 
            "id": 488673261, 
            "name": "EphysInstantaneousThresholdThumbnail", 
            "updated_at": "2015-10-14T10:14:10-07:00"
          }, 
          "well_known_file_type_id": 488673261, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "application/x-hdf; charset=binary", 
          "created_at": "2015-01-29T22:38:04-08:00", 
          "file_source_id": null, 
          "filename": "Nr5a1-Cre;Ai14(IVSCC)-169248.04.02.orca", 
          "id": 324326353, 
          "published_at": null, 
          "size": null, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2015-01-29T22:38:04-08:00", 
          "well_known_file_type": {
            "created_at": "2014-11-20T19:45:24-08:00", 
            "id": 311813285, 
            "name": "ORCA", 
            "updated_at": "2014-11-20T19:45:24-08:00"
          }, 
          "well_known_file_type_id": 311813285, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "application/x-hdf; charset=binary", 
          "created_at": "2015-04-28T20:08:07-07:00", 
          "file_source_id": null, 
          "filename": "318733869.nwb", 
          "id": 475693115, 
          "published_at": null, 
          "size": null, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2015-04-28T20:08:07-07:00", 
          "well_known_file_type": {
            "created_at": "2015-04-16T14:09:40-07:00", 
            "id": 475137571, 
            "name": "NWB", 
            "updated_at": "2015-04-16T14:09:40-07:00"
          }, 
          "well_known_file_type_id": 475137571, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "application/x-hdf; charset=binary", 
          "created_at": "2015-11-16T14:00:58-08:00", 
          "file_source_id": null, 
          "filename": "318733869_ephys.nwb", 
          "id": 491198860, 
          "published_at": "2015-05-14", 
          "size": null, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2015-11-16T14:00:58-08:00", 
          "well_known_file_type": {
            "created_at": "2015-07-14T14:11:16-07:00", 
            "id": 481007198, 
            "name": "NWBDownload", 
            "updated_at": "2015-07-14T14:11:16-07:00"
          }, 
          "well_known_file_type_id": 481007198, 
          "workflow_state": null
        }, 
        {
          "attachable_id": 318733869, 
          "attachable_type": "EphysRoiResult", 
          "content_type": "application/x-hdf; charset=binary", 
          "created_at": "2015-11-16T14:00:58-08:00", 
          "file_source_id": null, 
          "filename": "318733869_uncompressed.nwb", 
          "id": 491198863, 
          "published_at": "2015-05-14", 
          "size": null, 
          "storage_directory": "/projects/mousecelltypes/vol1/prod128/Ephys_Specimen_Roi_plan_318733869/", 
          "updated_at": "2015-11-16T14:00:58-08:00", 
          "well_known_file_type": {
            "created_at": "2015-06-09T15:21:33-07:00", 
            "id": 478840678, 
            "name": "NWBUncompressed", 
            "updated_at": "2015-06-09T15:21:33-07:00"
          }, 
          "well_known_file_type_id": 478840678, 
          "workflow_state": null
        }
      ], 
      "workflow_state": "manual_passed"
    }, 
    "ephys_roi_result_id": 318733869, 
    "ephys_sweeps": [
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[7]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338141, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338141, 
        "ephys_sweep_tags": [], 
        "id": 396323610, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0356589965522289, 
        "post_vm_mv": -73.690673828125, 
        "pre_noise_rms_mv": 0.0380660705268383, 
        "pre_vm_mv": -73.6825790405273, 
        "slow_noise_rms_mv": 0.181937232613564, 
        "slow_vm_mv": -73.6825790405273, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.072995, 
        "stimulus_interval": 0.035, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 87, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.00809478759765625, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[6]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338137, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338137, 
        "ephys_sweep_tags": [], 
        "id": 396323608, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0338849276304245, 
        "post_vm_mv": -73.9566116333008, 
        "pre_noise_rms_mv": 0.0396845452487469, 
        "pre_vm_mv": -74.0519638061523, 
        "slow_noise_rms_mv": 0.18243981897831, 
        "slow_vm_mv": -74.0519638061523, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.064995, 
        "stimulus_interval": 0.031, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 86, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.0953521728515625, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[5]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338131, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338131, 
        "ephys_sweep_tags": [], 
        "id": 396323606, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0375229977071285, 
        "post_vm_mv": -73.8176422119141, 
        "pre_noise_rms_mv": 0.0395662672817707, 
        "pre_vm_mv": -73.6396636962891, 
        "slow_noise_rms_mv": 0.279655814170837, 
        "slow_vm_mv": -73.6396636962891, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.056995, 
        "stimulus_interval": 0.027, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 85, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.177978515625, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[6]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338137, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338137, 
        "ephys_sweep_tags": [], 
        "id": 396323584, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0425188019871712, 
        "post_vm_mv": -73.2127685546875, 
        "pre_noise_rms_mv": 0.0361235067248344, 
        "pre_vm_mv": -73.4286575317383, 
        "slow_noise_rms_mv": 0.178375139832497, 
        "slow_vm_mv": -73.4286575317383, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.064995, 
        "stimulus_interval": 0.031, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 78, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.215888977050781, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[5]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338131, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338131, 
        "ephys_sweep_tags": [], 
        "id": 396323582, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0382817052304745, 
        "post_vm_mv": -73.6458282470703, 
        "pre_noise_rms_mv": 0.0396089442074299, 
        "pre_vm_mv": -73.4182739257812, 
        "slow_noise_rms_mv": 0.228058248758316, 
        "slow_vm_mv": -73.4182739257812, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.056995, 
        "stimulus_interval": 0.027, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 77, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.227554321289062, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[4]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338127, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338127, 
        "ephys_sweep_tags": [], 
        "id": 396323580, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0367705412209034, 
        "post_vm_mv": -73.3423385620117, 
        "pre_noise_rms_mv": 0.0410844683647156, 
        "pre_vm_mv": -73.2242660522461, 
        "slow_noise_rms_mv": 0.295760065317154, 
        "slow_vm_mv": -73.2242660522461, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.048995, 
        "stimulus_interval": 0.023, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 76, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.118072509765625, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[3]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338123, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338123, 
        "ephys_sweep_tags": [], 
        "id": 396323577, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0412763878703117, 
        "post_vm_mv": -73.5490570068359, 
        "pre_noise_rms_mv": 0.0366058796644211, 
        "pre_vm_mv": -73.0638885498047, 
        "slow_noise_rms_mv": 0.171120792627335, 
        "slow_vm_mv": -73.0638885498047, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.040995, 
        "stimulus_interval": 0.019, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 75, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.48516845703125, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[2]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338119, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338119, 
        "ephys_sweep_tags": [], 
        "id": 396323573, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0449937023222446, 
        "post_vm_mv": -73.4591293334961, 
        "pre_noise_rms_mv": 0.0359138548374176, 
        "pre_vm_mv": -73.5854873657227, 
        "slow_noise_rms_mv": 0.337626546621323, 
        "slow_vm_mv": -73.5854873657227, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.032995, 
        "stimulus_interval": 0.015, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 74, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.126358032226562, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[1]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338115, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338115, 
        "ephys_sweep_tags": [], 
        "id": 396323570, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0407821051776409, 
        "post_vm_mv": -73.443000793457, 
        "pre_noise_rms_mv": 0.050120122730732, 
        "pre_vm_mv": -73.2168655395508, 
        "slow_noise_rms_mv": 0.144578456878662, 
        "slow_vm_mv": -73.2168655395508, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.024995, 
        "stimulus_interval": 0.011, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 73, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.22613525390625, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:15-08:00", 
          "description": "C2SSTRIPLE141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338111, 
          "updated_at": "2015-01-29T22:53:15-08:00"
        }, 
        "ephys_stimulus_id": 324338111, 
        "ephys_sweep_tags": [], 
        "id": 396323568, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0424049384891987, 
        "post_vm_mv": -73.3690872192383, 
        "pre_noise_rms_mv": 0.040862325578928, 
        "pre_vm_mv": -73.4466323852539, 
        "slow_noise_rms_mv": 0.147407010197639, 
        "slow_vm_mv": -73.4466323852539, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.016995, 
        "stimulus_interval": 0.007, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 72, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.077545166015625, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[7]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338141, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338141, 
        "ephys_sweep_tags": [], 
        "id": 396323566, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0450072474777699, 
        "post_vm_mv": -73.5418701171875, 
        "pre_noise_rms_mv": 0.0406083464622498, 
        "pre_vm_mv": -73.3598327636719, 
        "slow_noise_rms_mv": 0.273215651512146, 
        "slow_vm_mv": -73.3598327636719, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.072995, 
        "stimulus_interval": 0.035, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 71, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.182037353515625, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[6]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338137, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338137, 
        "ephys_sweep_tags": [], 
        "id": 396323564, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0368791185319424, 
        "post_vm_mv": -73.338981628418, 
        "pre_noise_rms_mv": 0.0398431569337845, 
        "pre_vm_mv": -73.1990432739258, 
        "slow_noise_rms_mv": 0.220605164766312, 
        "slow_vm_mv": -73.1990432739258, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.064995, 
        "stimulus_interval": 0.031, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 70, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.139938354492188, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[5]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338131, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338131, 
        "ephys_sweep_tags": [], 
        "id": 396323558, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0424865819513798, 
        "post_vm_mv": -73.4840774536133, 
        "pre_noise_rms_mv": 0.0386396385729313, 
        "pre_vm_mv": -73.3247375488281, 
        "slow_noise_rms_mv": 0.158759966492653, 
        "slow_vm_mv": -73.3247375488281, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.056995, 
        "stimulus_interval": 0.027, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 69, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.159339904785156, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[4]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338127, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338127, 
        "ephys_sweep_tags": [], 
        "id": 396323556, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0348531827330589, 
        "post_vm_mv": -73.8137969970703, 
        "pre_noise_rms_mv": 0.0357643850147724, 
        "pre_vm_mv": -73.6382293701172, 
        "slow_noise_rms_mv": 0.243798926472664, 
        "slow_vm_mv": -73.6382293701172, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.048995, 
        "stimulus_interval": 0.023, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 68, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.175567626953125, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:18:36-08:00", 
          "description": "C2NSRMPRHE141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305829, 
            "name": "Ramp to Rheobase", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305829, 
          "id": 324321519, 
          "updated_at": "2015-01-29T22:18:36-08:00"
        }, 
        "ephys_stimulus_id": 324321519, 
        "ephys_sweep_tags": [], 
        "id": 396323540, 
        "leak_pa": -12.028151512146, 
        "num_spikes": 59, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0456047654151917, 
        "post_vm_mv": -74.8246002197266, 
        "pre_noise_rms_mv": 0.0413780510425568, 
        "pre_vm_mv": -73.5059814453125, 
        "slow_noise_rms_mv": 0.223160579800606, 
        "slow_vm_mv": -73.5059814453125, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 255.750004507505, 
        "stimulus_duration": 29.999975, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 2.02002, 
        "stimulus_units": "Amps", 
        "sweep_number": 62, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 1.31861877441406, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:18:36-08:00", 
          "description": "C2NSRMPRHE141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305829, 
            "name": "Ramp to Rheobase", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305829, 
          "id": 324321519, 
          "updated_at": "2015-01-29T22:18:36-08:00"
        }, 
        "ephys_stimulus_id": 324321519, 
        "ephys_sweep_tags": [], 
        "id": 396323513, 
        "leak_pa": -14.9712104797363, 
        "num_spikes": 80, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0416221991181374, 
        "post_vm_mv": -74.8126831054688, 
        "pre_noise_rms_mv": 0.0385609716176987, 
        "pre_vm_mv": -72.8345184326172, 
        "slow_noise_rms_mv": 0.207559898495674, 
        "slow_vm_mv": -72.8345184326172, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 255.750004507505, 
        "stimulus_duration": 29.999975, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 2.02002, 
        "stimulus_units": "Amps", 
        "sweep_number": 61, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 1.97816467285156, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.8, 
          "created_at": "2015-01-29T22:13:02-08:00", 
          "description": "C2SQRHELNG141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305801, 
            "name": "Square - 2s Suprathreshold", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305801, 
          "id": 324318810, 
          "updated_at": "2015-01-29T22:13:02-08:00"
        }, 
        "ephys_stimulus_id": 324318810, 
        "ephys_sweep_tags": [], 
        "id": 396323510, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 16, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0378606133162975, 
        "post_vm_mv": -73.6508865356445, 
        "pre_noise_rms_mv": 0.0414417758584023, 
        "pre_vm_mv": -72.4496307373047, 
        "slow_noise_rms_mv": 0.236516460776329, 
        "slow_vm_mv": -72.4496307373047, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 180.000001015479, 
        "stimulus_duration": 1.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 60, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 1.20125579833984, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:33:09-08:00", 
          "description": "C1RP25PR1S141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:09-08:00", 
            "id": 324305749, 
            "name": "Ramp", 
            "updated_at": "2015-01-29T21:33:09-08:00"
          }, 
          "ephys_stimulus_type_id": 324305749, 
          "id": 324305751, 
          "updated_at": "2015-01-29T21:33:09-08:00"
        }, 
        "ephys_stimulus_id": 324305751, 
        "ephys_sweep_tags": [], 
        "id": 396323508, 
        "leak_pa": 0.0, 
        "num_spikes": 17, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0, 
        "post_vm_mv": 0.0, 
        "pre_noise_rms_mv": 0.0400054045021534, 
        "pre_vm_mv": -72.8523635864258, 
        "slow_noise_rms_mv": 0.152621924877167, 
        "slow_vm_mv": -72.8523635864258, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 800.000010681146, 
        "stimulus_duration": 31.997495, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.0225, 
        "stimulus_units": "Amps", 
        "sweep_number": 6, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.0, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.8, 
          "created_at": "2015-01-29T22:13:02-08:00", 
          "description": "C2SQRHELNG141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305801, 
            "name": "Square - 2s Suprathreshold", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305801, 
          "id": 324318810, 
          "updated_at": "2015-01-29T22:13:02-08:00"
        }, 
        "ephys_stimulus_id": 324318810, 
        "ephys_sweep_tags": [], 
        "id": 396323506, 
        "leak_pa": -12.028151512146, 
        "num_spikes": 13, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0390940234065056, 
        "post_vm_mv": -73.9824295043945, 
        "pre_noise_rms_mv": 0.0369674041867256, 
        "pre_vm_mv": -73.267463684082, 
        "slow_noise_rms_mv": 0.177908077836037, 
        "slow_vm_mv": -73.267463684082, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 180.000001015479, 
        "stimulus_duration": 1.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 59, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.7149658203125, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.8, 
          "created_at": "2015-01-29T22:13:02-08:00", 
          "description": "C2SQRHELNG141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305801, 
            "name": "Square - 2s Suprathreshold", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305801, 
          "id": 324318810, 
          "updated_at": "2015-01-29T22:13:02-08:00"
        }, 
        "ephys_stimulus_id": 324318810, 
        "ephys_sweep_tags": [], 
        "id": 396323504, 
        "leak_pa": -14.9712104797363, 
        "num_spikes": 14, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0380340926349163, 
        "post_vm_mv": -74.5748748779297, 
        "pre_noise_rms_mv": 0.0393643826246262, 
        "pre_vm_mv": -73.4411468505859, 
        "slow_noise_rms_mv": 0.236093729734421, 
        "slow_vm_mv": -73.4411468505859, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 180.000001015479, 
        "stimulus_duration": 1.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 58, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 1.13372802734375, 
        "workflow_state": "auto_failed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.4, 
          "created_at": "2015-01-29T22:13:02-08:00", 
          "description": "C2SQRHELNG141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305801, 
            "name": "Square - 2s Suprathreshold", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305801, 
          "id": 324318800, 
          "updated_at": "2015-01-29T22:13:02-08:00"
        }, 
        "ephys_stimulus_id": 324318800, 
        "ephys_sweep_tags": [], 
        "id": 396323501, 
        "leak_pa": -14.9712104797363, 
        "num_spikes": 7, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0387220717966557, 
        "post_vm_mv": -74.2508087158203, 
        "pre_noise_rms_mv": 0.0425904989242554, 
        "pre_vm_mv": -73.5337600708008, 
        "slow_noise_rms_mv": 0.274801641702652, 
        "slow_vm_mv": -73.5337600708008, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 139.999997705864, 
        "stimulus_duration": 1.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 57, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.717048645019531, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.4, 
          "created_at": "2015-01-29T22:13:02-08:00", 
          "description": "C2SQRHELNG141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305801, 
            "name": "Square - 2s Suprathreshold", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305801, 
          "id": 324318800, 
          "updated_at": "2015-01-29T22:13:02-08:00"
        }, 
        "ephys_stimulus_id": 324318800, 
        "ephys_sweep_tags": [], 
        "id": 396323497, 
        "leak_pa": -14.9712104797363, 
        "num_spikes": 7, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0402530431747437, 
        "post_vm_mv": -74.1909713745117, 
        "pre_noise_rms_mv": 0.0405836701393127, 
        "pre_vm_mv": -73.9143676757812, 
        "slow_noise_rms_mv": 0.143637120723724, 
        "slow_vm_mv": -73.9143676757812, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 139.999997705864, 
        "stimulus_duration": 1.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 56, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.276603698730469, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.4, 
          "created_at": "2015-01-29T22:13:02-08:00", 
          "description": "C2SQRHELNG141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305801, 
            "name": "Square - 2s Suprathreshold", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305801, 
          "id": 324318800, 
          "updated_at": "2015-01-29T22:13:02-08:00"
        }, 
        "ephys_stimulus_id": 324318800, 
        "ephys_sweep_tags": [], 
        "id": 396323494, 
        "leak_pa": -14.9712104797363, 
        "num_spikes": 6, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0446703843772411, 
        "post_vm_mv": -74.1566619873047, 
        "pre_noise_rms_mv": 0.0386499427258968, 
        "pre_vm_mv": -73.4279251098633, 
        "slow_noise_rms_mv": 0.285361737012863, 
        "slow_vm_mv": -73.4279251098633, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 139.999997705864, 
        "stimulus_duration": 1.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 55, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.728736877441406, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:13:24-08:00", 
          "description": "C2SQRHELNG141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305801, 
            "name": "Square - 2s Suprathreshold", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305801, 
          "id": 324320270, 
          "updated_at": "2015-01-29T22:13:24-08:00"
        }, 
        "ephys_stimulus_id": 324320270, 
        "ephys_sweep_tags": [], 
        "id": 396323492, 
        "leak_pa": -14.9712104797363, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0452239476144314, 
        "post_vm_mv": -73.4174270629883, 
        "pre_noise_rms_mv": 0.044888649135828, 
        "pre_vm_mv": -72.06005859375, 
        "slow_noise_rms_mv": 0.587946355342865, 
        "slow_vm_mv": -72.06005859375, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 100.000001335143, 
        "stimulus_duration": 1.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 54, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 1.35736846923828, 
        "workflow_state": "auto_failed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:13:24-08:00", 
          "description": "C2SQRHELNG141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305801, 
            "name": "Square - 2s Suprathreshold", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305801, 
          "id": 324320270, 
          "updated_at": "2015-01-29T22:13:24-08:00"
        }, 
        "ephys_stimulus_id": 324320270, 
        "ephys_sweep_tags": [], 
        "id": 396323490, 
        "leak_pa": -14.9712104797363, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.039005272090435, 
        "post_vm_mv": -73.7748794555664, 
        "pre_noise_rms_mv": 0.0346203036606312, 
        "pre_vm_mv": -73.50634765625, 
        "slow_noise_rms_mv": 0.164012134075165, 
        "slow_vm_mv": -73.50634765625, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 100.000001335143, 
        "stimulus_duration": 1.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 53, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.268531799316406, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:13:24-08:00", 
          "description": "C2SQRHELNG141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305801, 
            "name": "Square - 2s Suprathreshold", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305801, 
          "id": 324320270, 
          "updated_at": "2015-01-29T22:13:24-08:00"
        }, 
        "ephys_stimulus_id": 324320270, 
        "ephys_sweep_tags": [], 
        "id": 396323488, 
        "leak_pa": -14.9712104797363, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0385660864412785, 
        "post_vm_mv": -73.2594223022461, 
        "pre_noise_rms_mv": 0.0388743877410889, 
        "pre_vm_mv": -72.9849166870117, 
        "slow_noise_rms_mv": 0.202888786792755, 
        "slow_vm_mv": -72.9849166870117, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 100.000001335143, 
        "stimulus_duration": 1.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 52, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.274505615234375, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:33:09-08:00", 
          "description": "C1RP25PR1S141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:09-08:00", 
            "id": 324305749, 
            "name": "Ramp", 
            "updated_at": "2015-01-29T21:33:09-08:00"
          }, 
          "ephys_stimulus_type_id": 324305749, 
          "id": 324305751, 
          "updated_at": "2015-01-29T21:33:09-08:00"
        }, 
        "ephys_stimulus_id": 324305751, 
        "ephys_sweep_tags": [], 
        "id": 396323480, 
        "leak_pa": 0.0, 
        "num_spikes": 8, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0, 
        "post_vm_mv": 0.0, 
        "pre_noise_rms_mv": 0.0415542982518673, 
        "pre_vm_mv": -73.3303985595703, 
        "slow_noise_rms_mv": 0.0891085341572762, 
        "slow_vm_mv": -73.3303985595703, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 800.000010681146, 
        "stimulus_duration": 31.997495, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.0225, 
        "stimulus_units": "Amps", 
        "sweep_number": 5, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.0, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 0.9, 
          "created_at": "2015-01-29T22:03:09-08:00", 
          "description": "C1NSSEED_2141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:09-08:00", 
            "id": 324305777, 
            "name": "Noise 2", 
            "updated_at": "2015-01-29T21:33:09-08:00"
          }, 
          "ephys_stimulus_type_id": 324305777, 
          "id": 324312198, 
          "updated_at": "2015-01-29T22:03:09-08:00"
        }, 
        "ephys_stimulus_id": 324312198, 
        "ephys_sweep_tags": [], 
        "id": 396323475, 
        "leak_pa": -12.028151512146, 
        "num_spikes": 23, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.037276167422533, 
        "post_vm_mv": -74.3984375, 
        "pre_noise_rms_mv": 0.0387932918965816, 
        "pre_vm_mv": -73.5012130737305, 
        "slow_noise_rms_mv": 0.178911685943604, 
        "slow_vm_mv": -73.5012130737305, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 212.249995357183, 
        "stimulus_duration": 18.999995, 
        "stimulus_interval": 8.0, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 48, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.897224426269531, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 0.9, 
          "created_at": "2015-01-29T22:03:09-08:00", 
          "description": "C1NSSEED_1141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:09-08:00", 
            "id": 324305771, 
            "name": "Noise 1", 
            "updated_at": "2015-01-29T21:33:09-08:00"
          }, 
          "ephys_stimulus_type_id": 324305771, 
          "id": 324312194, 
          "updated_at": "2015-01-29T22:03:09-08:00"
        }, 
        "ephys_stimulus_id": 324312194, 
        "ephys_sweep_tags": [], 
        "id": 396323471, 
        "leak_pa": -12.028151512146, 
        "num_spikes": 26, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0358653254806995, 
        "post_vm_mv": -73.3166122436523, 
        "pre_noise_rms_mv": 0.0393880866467953, 
        "pre_vm_mv": -72.8331680297852, 
        "slow_noise_rms_mv": 0.148469358682632, 
        "slow_vm_mv": -72.8331680297852, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 208.62500615948, 
        "stimulus_duration": 18.999995, 
        "stimulus_interval": 8.0, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 47, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.483444213867188, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 0.9, 
          "created_at": "2015-01-29T22:03:09-08:00", 
          "description": "C1NSSEED_2141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:09-08:00", 
            "id": 324305777, 
            "name": "Noise 2", 
            "updated_at": "2015-01-29T21:33:09-08:00"
          }, 
          "ephys_stimulus_type_id": 324305777, 
          "id": 324312198, 
          "updated_at": "2015-01-29T22:03:09-08:00"
        }, 
        "ephys_stimulus_id": 324312198, 
        "ephys_sweep_tags": [], 
        "id": 396323468, 
        "leak_pa": -14.0115156173706, 
        "num_spikes": 25, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.040932834148407, 
        "post_vm_mv": -74.2956924438477, 
        "pre_noise_rms_mv": 0.0322319604456425, 
        "pre_vm_mv": -72.9757995605469, 
        "slow_noise_rms_mv": 0.173675671219826, 
        "slow_vm_mv": -72.9757995605469, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 212.249995357183, 
        "stimulus_duration": 18.999995, 
        "stimulus_interval": 8.0, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 46, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 1.31989288330078, 
        "workflow_state": "auto_failed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 0.9, 
          "created_at": "2015-01-29T22:03:09-08:00", 
          "description": "C1NSSEED_1141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:09-08:00", 
            "id": 324305771, 
            "name": "Noise 1", 
            "updated_at": "2015-01-29T21:33:09-08:00"
          }, 
          "ephys_stimulus_type_id": 324305771, 
          "id": 324312194, 
          "updated_at": "2015-01-29T22:03:09-08:00"
        }, 
        "ephys_stimulus_id": 324312194, 
        "ephys_sweep_tags": [], 
        "id": 396323466, 
        "leak_pa": -14.0115156173706, 
        "num_spikes": 23, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0391243994235992, 
        "post_vm_mv": -73.6055221557617, 
        "pre_noise_rms_mv": 0.0382960624992847, 
        "pre_vm_mv": -73.330451965332, 
        "slow_noise_rms_mv": 0.197248265147209, 
        "slow_vm_mv": -73.330451965332, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 208.62500615948, 
        "stimulus_duration": 18.999995, 
        "stimulus_interval": 8.0, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 45, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.275070190429688, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 0.9, 
          "created_at": "2015-01-29T22:03:09-08:00", 
          "description": "C1NSSEED_2141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:09-08:00", 
            "id": 324305777, 
            "name": "Noise 2", 
            "updated_at": "2015-01-29T21:33:09-08:00"
          }, 
          "ephys_stimulus_type_id": 324305777, 
          "id": 324312198, 
          "updated_at": "2015-01-29T22:03:09-08:00"
        }, 
        "ephys_stimulus_id": 324312198, 
        "ephys_sweep_tags": [], 
        "id": 396323464, 
        "leak_pa": -14.0115156173706, 
        "num_spikes": 31, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0463137738406658, 
        "post_vm_mv": -72.9353485107422, 
        "pre_noise_rms_mv": 0.036839384585619, 
        "pre_vm_mv": -73.1283111572266, 
        "slow_noise_rms_mv": 0.166697889566422, 
        "slow_vm_mv": -73.1283111572266, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 212.249995357183, 
        "stimulus_duration": 18.999995, 
        "stimulus_interval": 8.0, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 44, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.192962646484375, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 0.9, 
          "created_at": "2015-01-29T22:03:09-08:00", 
          "description": "C1NSSEED_1141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:09-08:00", 
            "id": 324305771, 
            "name": "Noise 1", 
            "updated_at": "2015-01-29T21:33:09-08:00"
          }, 
          "ephys_stimulus_type_id": 324305771, 
          "id": 324312194, 
          "updated_at": "2015-01-29T22:03:09-08:00"
        }, 
        "ephys_stimulus_id": 324312194, 
        "ephys_sweep_tags": [], 
        "id": 396323462, 
        "leak_pa": -14.0115156173706, 
        "num_spikes": 32, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0393371880054474, 
        "post_vm_mv": -73.7409057617188, 
        "pre_noise_rms_mv": 0.0401734113693237, 
        "pre_vm_mv": -73.3647308349609, 
        "slow_noise_rms_mv": 0.203217536211014, 
        "slow_vm_mv": -73.3647308349609, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 208.62500615948, 
        "stimulus_duration": 18.999995, 
        "stimulus_interval": 8.0, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 43, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.376174926757812, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 0.9, 
          "created_at": "2015-01-29T22:18:33-08:00", 
          "description": "C1LSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324321353, 
          "updated_at": "2015-01-29T22:18:33-08:00"
        }, 
        "ephys_stimulus_id": 324321353, 
        "ephys_sweep_tags": [], 
        "id": 396323459, 
        "leak_pa": -12.028151512146, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0324449688196182, 
        "post_vm_mv": -72.5376968383789, 
        "pre_noise_rms_mv": 0.0383681282401085, 
        "pre_vm_mv": -72.81201171875, 
        "slow_noise_rms_mv": 0.204954192042351, 
        "slow_vm_mv": -72.81201171875, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 90.0000005077395, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 42, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.274314880371094, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:07-08:00", 
          "description": "C1LSCOARSE141203[13]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310907, 
          "updated_at": "2015-01-29T21:58:07-08:00"
        }, 
        "ephys_stimulus_id": 324310907, 
        "ephys_sweep_tags": [], 
        "id": 396323441, 
        "leak_pa": -7.99744129180908, 
        "num_spikes": 18, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0342174433171749, 
        "post_vm_mv": -72.9227523803711, 
        "pre_noise_rms_mv": 0.0359137654304504, 
        "pre_vm_mv": -72.2949981689453, 
        "slow_noise_rms_mv": 0.146864429116249, 
        "slow_vm_mv": -72.2949981689453, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 149.999998533268, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 35, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.627754211425781, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:07-08:00", 
          "description": "C1LSCOARSE141203[12]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310903, 
          "updated_at": "2015-01-29T21:58:07-08:00"
        }, 
        "ephys_stimulus_id": 324310903, 
        "ephys_sweep_tags": [], 
        "id": 396323439, 
        "leak_pa": -7.99744129180908, 
        "num_spikes": 14, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0305392686277628, 
        "post_vm_mv": -72.4750366210938, 
        "pre_noise_rms_mv": 0.04054119810462, 
        "pre_vm_mv": -72.5203247070312, 
        "slow_noise_rms_mv": 0.177269637584686, 
        "slow_vm_mv": -72.5203247070312, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 129.99999687846, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 34, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.0452880859375, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1LSCOARSE141203[11]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310899, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310899, 
        "ephys_sweep_tags": [], 
        "id": 396323437, 
        "leak_pa": -7.99744129180908, 
        "num_spikes": 10, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0409046038985252, 
        "post_vm_mv": -72.9968566894531, 
        "pre_noise_rms_mv": 0.0379738472402096, 
        "pre_vm_mv": -73.0722732543945, 
        "slow_noise_rms_mv": 0.136501178145409, 
        "slow_vm_mv": -73.0722732543945, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 110.000002162547, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 33, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.0754165649414062, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1LSCOARSE141203[10]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310891, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310891, 
        "ephys_sweep_tags": [], 
        "id": 396323435, 
        "leak_pa": -7.99744129180908, 
        "num_spikes": 4, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0394844189286232, 
        "post_vm_mv": -72.8281784057617, 
        "pre_noise_rms_mv": 0.0399150885641575, 
        "pre_vm_mv": -72.929557800293, 
        "slow_noise_rms_mv": 0.221334338188171, 
        "slow_vm_mv": -72.929557800293, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 90.0000005077395, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 32, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.10137939453125, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1LSCOARSE141203[9]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310887, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310887, 
        "ephys_sweep_tags": [], 
        "id": 396323433, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 2, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.038056381046772, 
        "post_vm_mv": -71.3184967041016, 
        "pre_noise_rms_mv": 0.039063710719347, 
        "pre_vm_mv": -71.836311340332, 
        "slow_noise_rms_mv": 0.288136333227158, 
        "slow_vm_mv": -71.836311340332, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 69.9999988529321, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 31, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.517814636230469, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T21:58:05-08:00", 
          "description": "C1SSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310797, 
          "updated_at": "2015-01-29T21:58:05-08:00"
        }, 
        "ephys_stimulus_id": 324310797, 
        "ephys_sweep_tags": [], 
        "id": 396323380, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0418095923960209, 
        "post_vm_mv": -70.9423675537109, 
        "pre_noise_rms_mv": 0.0406226739287376, 
        "pre_vm_mv": -72.5032501220703, 
        "slow_noise_rms_mv": 0.300107926130295, 
        "slow_vm_mv": -72.5032501220703, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 20, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 1.56088256835938, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T21:58:05-08:00", 
          "description": "C1SSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310797, 
          "updated_at": "2015-01-29T21:58:05-08:00"
        }, 
        "ephys_stimulus_id": 324310797, 
        "ephys_sweep_tags": [], 
        "id": 396323375, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0419786125421524, 
        "post_vm_mv": -70.9460601806641, 
        "pre_noise_rms_mv": 0.0391221158206463, 
        "pre_vm_mv": -72.9047927856445, 
        "slow_noise_rms_mv": 0.138547375798225, 
        "slow_vm_mv": -72.9047927856445, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 19, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 1.95873260498047, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:05-08:00", 
          "description": "C1SSCOARSE141203[5]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310725, 
          "updated_at": "2015-01-29T21:58:05-08:00"
        }, 
        "ephys_stimulus_id": 324310725, 
        "ephys_sweep_tags": [], 
        "id": 396323356, 
        "leak_pa": -1.9833652973175, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0374195724725723, 
        "post_vm_mv": -71.2353363037109, 
        "pre_noise_rms_mv": 0.0365947894752026, 
        "pre_vm_mv": -72.5235977172852, 
        "slow_noise_rms_mv": 0.45252200961113, 
        "slow_vm_mv": -72.5235977172852, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 12, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 1.28826141357422, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.6, 
          "created_at": "2015-01-29T22:39:35-08:00", 
          "description": "C2SSHM80FN141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324331247, 
          "updated_at": "2015-01-29T22:39:35-08:00"
        }, 
        "ephys_stimulus_id": 324331247, 
        "ephys_sweep_tags": [], 
        "id": 396323338, 
        "leak_pa": -39.9872055053711, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0446453168988228, 
        "post_vm_mv": -79.0671920776367, 
        "pre_noise_rms_mv": 0.0347333513200283, 
        "pre_vm_mv": -80.5915985107422, 
        "slow_noise_rms_mv": 0.0647063180804253, 
        "slow_vm_mv": -80.5915985107422, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 659.999999097494, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 101, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 1.52440643310547, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.6, 
          "created_at": "2015-01-29T22:39:35-08:00", 
          "description": "C2SSHM80FN141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324331247, 
          "updated_at": "2015-01-29T22:39:35-08:00"
        }, 
        "ephys_stimulus_id": 324331247, 
        "ephys_sweep_tags": [], 
        "id": 396323336, 
        "leak_pa": -39.9872055053711, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0389018692076206, 
        "post_vm_mv": -79.1583862304688, 
        "pre_noise_rms_mv": 0.0407516062259674, 
        "pre_vm_mv": -80.5845718383789, 
        "slow_noise_rms_mv": 0.0986397936940193, 
        "slow_vm_mv": -80.5845718383789, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 659.999999097494, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 100, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 1.42618560791016, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:03:11-08:00", 
          "description": "C2SSHM80CS141203[6]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324312332, 
          "updated_at": "2015-01-29T22:03:11-08:00"
        }, 
        "ephys_stimulus_id": 324312332, 
        "ephys_sweep_tags": [], 
        "id": 396323638, 
        "leak_pa": -38.0038414001465, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0389880537986755, 
        "post_vm_mv": -78.6123733520508, 
        "pre_noise_rms_mv": 0.0441122837364674, 
        "pre_vm_mv": -80.0253067016602, 
        "slow_noise_rms_mv": 0.094309464097023, 
        "slow_vm_mv": -80.0253067016602, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 700.000002407108, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 94, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 1.41293334960938, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:03:11-08:00", 
          "description": "C2SSHM80CS141203[5]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324312328, 
          "updated_at": "2015-01-29T22:03:11-08:00"
        }, 
        "ephys_stimulus_id": 324312328, 
        "ephys_sweep_tags": [], 
        "id": 396323629, 
        "leak_pa": -38.0038414001465, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0430155098438263, 
        "post_vm_mv": -79.0539321899414, 
        "pre_noise_rms_mv": 0.0329440385103226, 
        "pre_vm_mv": -80.0455932617188, 
        "slow_noise_rms_mv": 0.122686624526978, 
        "slow_vm_mv": -80.0455932617188, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 93, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.991661071777344, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:03:11-08:00", 
          "description": "C2SSHM80CS141203[4]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324312324, 
          "updated_at": "2015-01-29T22:03:11-08:00"
        }, 
        "ephys_stimulus_id": 324312324, 
        "ephys_sweep_tags": [], 
        "id": 396323626, 
        "leak_pa": -38.0038414001465, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0356684438884258, 
        "post_vm_mv": -79.3998413085938, 
        "pre_noise_rms_mv": 0.0402687676250935, 
        "pre_vm_mv": -80.3027191162109, 
        "slow_noise_rms_mv": 0.115302167832851, 
        "slow_vm_mv": -80.3027191162109, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 499.999985859034, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 92, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.902877807617188, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:03:11-08:00", 
          "description": "C2SSHM80CS141203[3]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324312320, 
          "updated_at": "2015-01-29T22:03:11-08:00"
        }, 
        "ephys_stimulus_id": 324312320, 
        "ephys_sweep_tags": [], 
        "id": 396323622, 
        "leak_pa": -38.0038414001465, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0391330868005753, 
        "post_vm_mv": -79.5965576171875, 
        "pre_noise_rms_mv": 0.0413624010980129, 
        "pre_vm_mv": -80.125602722168, 
        "slow_noise_rms_mv": 0.107317678630352, 
        "slow_vm_mv": -80.125602722168, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 400.000005340573, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 91, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.529045104980469, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:03:11-08:00", 
          "description": "C2SSHM80CS141203[2]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324312316, 
          "updated_at": "2015-01-29T22:03:11-08:00"
        }, 
        "ephys_stimulus_id": 324312316, 
        "ephys_sweep_tags": [], 
        "id": 396323620, 
        "leak_pa": -38.0038414001465, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0377324000000954, 
        "post_vm_mv": -79.911735534668, 
        "pre_noise_rms_mv": 0.0349041931331158, 
        "pre_vm_mv": -80.4060821533203, 
        "slow_noise_rms_mv": 0.0945140942931175, 
        "slow_vm_mv": -80.4060821533203, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 299.999997066536, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 90, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.494346618652344, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:07-08:00", 
          "description": "C1SSCOARSE141203[2]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310977, 
          "updated_at": "2015-01-29T21:58:07-08:00"
        }, 
        "ephys_stimulus_id": 324310977, 
        "ephys_sweep_tags": [], 
        "id": 396323618, 
        "leak_pa": -1.9833652973175, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0408011488616467, 
        "post_vm_mv": -73.0553512573242, 
        "pre_noise_rms_mv": 0.0391559898853302, 
        "pre_vm_mv": -74.0001449584961, 
        "slow_noise_rms_mv": 0.120690539479256, 
        "slow_vm_mv": -74.0001449584961, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 299.999997066536, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 9, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.944793701171875, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:03:11-08:00", 
          "description": "C2SSHM80CS141203[1]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324312312, 
          "updated_at": "2015-01-29T22:03:11-08:00"
        }, 
        "ephys_stimulus_id": 324312312, 
        "ephys_sweep_tags": [], 
        "id": 396323616, 
        "leak_pa": -38.0038414001465, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0408409088850021, 
        "post_vm_mv": -80.1303558349609, 
        "pre_noise_rms_mv": 0.0355650298297405, 
        "pre_vm_mv": -80.3894729614258, 
        "slow_noise_rms_mv": 0.11452279984951, 
        "slow_vm_mv": -80.3894729614258, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 200.000002670286, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 89, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.259117126464844, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:03:11-08:00", 
          "description": "C2SSHM80CS141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324312308, 
          "updated_at": "2015-01-29T22:03:11-08:00"
        }, 
        "ephys_stimulus_id": 324312308, 
        "ephys_sweep_tags": [], 
        "id": 396323614, 
        "leak_pa": -38.0038414001465, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0406179688870907, 
        "post_vm_mv": -80.2863159179688, 
        "pre_noise_rms_mv": 0.0393210723996162, 
        "pre_vm_mv": -80.4844284057617, 
        "slow_noise_rms_mv": 0.0890119299292564, 
        "slow_vm_mv": -80.4844284057617, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 100.000001335143, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 88, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.198112487792969, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:07-08:00", 
          "description": "C1SSCOARSE141203[1]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310939, 
          "updated_at": "2015-01-29T21:58:07-08:00"
        }, 
        "ephys_stimulus_id": 324310939, 
        "ephys_sweep_tags": [], 
        "id": 396323590, 
        "leak_pa": -1.9833652973175, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0383650623261929, 
        "post_vm_mv": -73.1851806640625, 
        "pre_noise_rms_mv": 0.0382653698325157, 
        "pre_vm_mv": -73.8074264526367, 
        "slow_noise_rms_mv": 0.151183858513832, 
        "slow_vm_mv": -73.8074264526367, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 200.000002670286, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 8, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.622245788574219, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1SSCOARSE141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310895, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310895, 
        "ephys_sweep_tags": [], 
        "id": 396323561, 
        "leak_pa": -1.9833652973175, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0413788892328739, 
        "post_vm_mv": -73.5245513916016, 
        "pre_noise_rms_mv": 0.0370076186954975, 
        "pre_vm_mv": -73.7009048461914, 
        "slow_noise_rms_mv": 0.159169092774391, 
        "slow_vm_mv": -73.7009048461914, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 100.000001335143, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 7, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.176353454589844, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 0.9, 
          "created_at": "2015-01-29T22:03:10-08:00", 
          "description": "C2SQRHELNG141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305801, 
            "name": "Square - 2s Suprathreshold", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305801, 
          "id": 324312282, 
          "updated_at": "2015-01-29T22:03:10-08:00"
        }, 
        "ephys_stimulus_id": 324312282, 
        "ephys_sweep_tags": [], 
        "id": 396323484, 
        "leak_pa": -14.9712104797363, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0418485626578331, 
        "post_vm_mv": -72.9245529174805, 
        "pre_noise_rms_mv": 0.0361919216811657, 
        "pre_vm_mv": -73.2862548828125, 
        "slow_noise_rms_mv": 0.257177621126175, 
        "slow_vm_mv": -73.2862548828125, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 90.0000005077395, 
        "stimulus_duration": 1.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 51, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.361701965332031, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:33:10-08:00", 
          "description": "C1SQCAPCHK141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305793, 
            "name": "Square - 0.5ms Subthreshold", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305793, 
          "id": 324305795, 
          "updated_at": "2015-01-29T21:33:10-08:00"
        }, 
        "ephys_stimulus_id": 324305795, 
        "ephys_sweep_tags": [], 
        "id": 396323482, 
        "leak_pa": -12.9878444671631, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0411632247269154, 
        "post_vm_mv": -72.7104568481445, 
        "pre_noise_rms_mv": 0.0371528267860413, 
        "pre_vm_mv": -72.659553527832, 
        "slow_noise_rms_mv": 0.483349621295929, 
        "slow_vm_mv": -72.659553527832, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": -200.000002670286, 
        "stimulus_duration": 7.218495, 
        "stimulus_interval": 0.2005, 
        "stimulus_start_time": 0.8215, 
        "stimulus_units": "Amps", 
        "sweep_number": 50, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.0509033203125, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:33:10-08:00", 
          "description": "C1SQCAPCHK141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305793, 
            "name": "Square - 0.5ms Subthreshold", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305793, 
          "id": 324305795, 
          "updated_at": "2015-01-29T21:33:10-08:00"
        }, 
        "ephys_stimulus_id": 324305795, 
        "ephys_sweep_tags": [], 
        "id": 396323478, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0388185605406761, 
        "post_vm_mv": -72.0395355224609, 
        "pre_noise_rms_mv": 0.0417257361114025, 
        "pre_vm_mv": -72.4330215454102, 
        "slow_noise_rms_mv": 0.372666418552399, 
        "slow_vm_mv": -72.4330215454102, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": -200.000002670286, 
        "stimulus_duration": 7.218495, 
        "stimulus_interval": 0.2005, 
        "stimulus_start_time": 0.8215, 
        "stimulus_units": "Amps", 
        "sweep_number": 49, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 0.393486022949219, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T21:58:05-08:00", 
          "description": "C1SSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310797, 
          "updated_at": "2015-01-29T21:58:05-08:00"
        }, 
        "ephys_stimulus_id": 324310797, 
        "ephys_sweep_tags": [], 
        "id": 396323371, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0344671793282032, 
        "post_vm_mv": -71.8193130493164, 
        "pre_noise_rms_mv": 0.0410925038158894, 
        "pre_vm_mv": -73.4055023193359, 
        "slow_noise_rms_mv": 0.126336947083473, 
        "slow_vm_mv": -73.4055023193359, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 18, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 1.58618927001953, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 5.9, 
          "created_at": "2015-01-29T21:58:05-08:00", 
          "description": "C1SSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310793, 
          "updated_at": "2015-01-29T21:58:05-08:00"
        }, 
        "ephys_stimulus_id": 324310793, 
        "ephys_sweep_tags": [], 
        "id": 396323364, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0410074442625046, 
        "post_vm_mv": -71.8181533813477, 
        "pre_noise_rms_mv": 0.0411656834185123, 
        "pre_vm_mv": -73.622314453125, 
        "slow_noise_rms_mv": 0.156922787427902, 
        "slow_vm_mv": -73.622314453125, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 589.999993305668, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 15, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 1.80416107177734, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 5.7, 
          "created_at": "2015-01-29T21:58:05-08:00", 
          "description": "C1SSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310761, 
          "updated_at": "2015-01-29T21:58:05-08:00"
        }, 
        "ephys_stimulus_id": 324310761, 
        "ephys_sweep_tags": [], 
        "id": 396323362, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0384085476398468, 
        "post_vm_mv": -72.1192779541016, 
        "pre_noise_rms_mv": 0.0402040779590607, 
        "pre_vm_mv": -73.5760116577148, 
        "slow_noise_rms_mv": 0.155263319611549, 
        "slow_vm_mv": -73.5760116577148, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 569.99999165086, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 14, 
        "updated_at": "2015-10-01T14:46:14-07:00", 
        "vm_delta_mv": 1.45673370361328, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 5.5, 
          "created_at": "2015-01-29T21:58:05-08:00", 
          "description": "C1SSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310729, 
          "updated_at": "2015-01-29T21:58:05-08:00"
        }, 
        "ephys_stimulus_id": 324310729, 
        "ephys_sweep_tags": [], 
        "id": 396323359, 
        "leak_pa": -1.9833652973175, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0412728488445282, 
        "post_vm_mv": -71.3994064331055, 
        "pre_noise_rms_mv": 0.0415022782981396, 
        "pre_vm_mv": -73.0148086547852, 
        "slow_noise_rms_mv": 0.316595315933228, 
        "slow_vm_mv": -73.0148086547852, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 549.999989996053, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 13, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 1.61540222167969, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:05-08:00", 
          "description": "C1SSCOARSE141203[4]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310721, 
          "updated_at": "2015-01-29T21:58:05-08:00"
        }, 
        "ephys_stimulus_id": 324310721, 
        "ephys_sweep_tags": [], 
        "id": 396323354, 
        "leak_pa": -1.9833652973175, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.036531925201416, 
        "post_vm_mv": -72.4496383666992, 
        "pre_noise_rms_mv": 0.0378439091145992, 
        "pre_vm_mv": -73.8205032348633, 
        "slow_noise_rms_mv": 0.171630755066872, 
        "slow_vm_mv": -73.8205032348633, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 499.999985859034, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 11, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 1.37086486816406, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 0.0, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:54:26-08:00", 
          "description": "EXTPBLWOUT141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305553, 
            "name": "Test", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305553, 
          "id": 324342566, 
          "updated_at": "2015-01-29T22:54:26-08:00"
        }, 
        "ephys_stimulus_id": 324342566, 
        "ephys_sweep_tags": [], 
        "id": 396323352, 
        "leak_pa": 0.0, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.040760412812233, 
        "post_vm_mv": 3.6764862537384, 
        "pre_noise_rms_mv": 0.0346001163125038, 
        "pre_vm_mv": 3.61960768699646, 
        "slow_noise_rms_mv": 0.0639408603310585, 
        "slow_vm_mv": 3.61960768699646, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 0.0, 
        "stimulus_duration": 0.0, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 0.0, 
        "stimulus_units": "Amps", 
        "sweep_number": 106, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.0568785667419434, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.6, 
          "created_at": "2015-01-29T22:39:35-08:00", 
          "description": "C2SSHM80FN141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324331247, 
          "updated_at": "2015-01-29T22:39:35-08:00"
        }, 
        "ephys_stimulus_id": 324331247, 
        "ephys_sweep_tags": [], 
        "id": 396323342, 
        "leak_pa": -39.9872055053711, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0422713197767735, 
        "post_vm_mv": -79.9530029296875, 
        "pre_noise_rms_mv": 0.0398413836956024, 
        "pre_vm_mv": -81.1040573120117, 
        "slow_noise_rms_mv": 0.0926332622766495, 
        "slow_vm_mv": -81.1040573120117, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 659.999999097494, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 103, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 1.15105438232422, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.6, 
          "created_at": "2015-01-29T22:39:35-08:00", 
          "description": "C2SSHM80FN141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324331247, 
          "updated_at": "2015-01-29T22:39:35-08:00"
        }, 
        "ephys_stimulus_id": 324331247, 
        "ephys_sweep_tags": [], 
        "id": 396323340, 
        "leak_pa": -39.9872055053711, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0415031537413597, 
        "post_vm_mv": -79.8756713867188, 
        "pre_noise_rms_mv": 0.0380181893706322, 
        "pre_vm_mv": -80.8639144897461, 
        "slow_noise_rms_mv": 0.10466530174017, 
        "slow_vm_mv": -80.8639144897461, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 659.999999097494, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 102, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.988243103027344, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:05-08:00", 
          "description": "C1SSCOARSE141203[3]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310717, 
          "updated_at": "2015-01-29T21:58:05-08:00"
        }, 
        "ephys_stimulus_id": 324310717, 
        "ephys_sweep_tags": [], 
        "id": 396323332, 
        "leak_pa": -1.9833652973175, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.039756141602993, 
        "post_vm_mv": -72.707145690918, 
        "pre_noise_rms_mv": 0.0399281866848469, 
        "pre_vm_mv": -73.8104858398438, 
        "slow_noise_rms_mv": 0.128612071275711, 
        "slow_vm_mv": -73.8104858398438, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 400.000005340573, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 10, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 1.10334014892578, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:06-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.6, 
          "created_at": "2015-01-29T22:39:35-08:00", 
          "description": "C2SSHM80FN141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324331247, 
          "updated_at": "2015-01-29T22:39:35-08:00"
        }, 
        "ephys_stimulus_id": 324331247, 
        "ephys_sweep_tags": [], 
        "id": 396323650, 
        "leak_pa": -39.9872055053711, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0323205776512623, 
        "post_vm_mv": -78.8193435668945, 
        "pre_noise_rms_mv": 0.0379347428679466, 
        "pre_vm_mv": -80.441047668457, 
        "slow_noise_rms_mv": 0.107122242450714, 
        "slow_vm_mv": -80.441047668457, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 659.999999097494, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 99, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 1.6217041015625, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:06-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.5, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSHM80FN141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324338195, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338195, 
        "ephys_sweep_tags": [], 
        "id": 396323647, 
        "leak_pa": -38.0038414001465, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0388760976493359, 
        "post_vm_mv": -79.2106018066406, 
        "pre_noise_rms_mv": 0.0425901263952255, 
        "pre_vm_mv": -80.2001800537109, 
        "slow_noise_rms_mv": 0.0826516449451447, 
        "slow_vm_mv": -80.2001800537109, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 649.99999827009, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 98, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.989578247070312, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:06-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.6, 
          "created_at": "2015-01-29T22:39:35-08:00", 
          "description": "C2SSHM80FN141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324331247, 
          "updated_at": "2015-01-29T22:39:35-08:00"
        }, 
        "ephys_stimulus_id": 324331247, 
        "ephys_sweep_tags": [], 
        "id": 396323644, 
        "leak_pa": -38.0038414001465, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.044320821762085, 
        "post_vm_mv": -78.6921768188477, 
        "pre_noise_rms_mv": 0.041571956127882, 
        "pre_vm_mv": -80.1463775634766, 
        "slow_noise_rms_mv": 0.136154472827911, 
        "slow_vm_mv": -80.1463775634766, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 659.999999097494, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 97, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 1.45420074462891, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:06-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.7, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSHM80FN141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324338199, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338199, 
        "ephys_sweep_tags": [], 
        "id": 396323642, 
        "leak_pa": -38.0038414001465, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0409424975514412, 
        "post_vm_mv": -78.6946182250977, 
        "pre_noise_rms_mv": 0.0409116707742214, 
        "pre_vm_mv": -79.6254272460938, 
        "slow_noise_rms_mv": 0.874026298522949, 
        "slow_vm_mv": -79.6254272460938, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 669.999999924897, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 96, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.930809020996094, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:06-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.5, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSHM80FN141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305569, 
            "name": "Short Square - Hold -80mV", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305569, 
          "id": 324338195, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338195, 
        "ephys_sweep_tags": [], 
        "id": 396323640, 
        "leak_pa": -38.0038414001465, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0357376858592033, 
        "post_vm_mv": -79.0951156616211, 
        "pre_noise_rms_mv": 0.0444186888635159, 
        "pre_vm_mv": -80.1246490478516, 
        "slow_noise_rms_mv": 0.110739670693874, 
        "slow_vm_mv": -80.1246490478516, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 649.99999827009, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 95, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 1.02953338623047, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:33:09-08:00", 
          "description": "C1RP25PR1S141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:09-08:00", 
            "id": 324305749, 
            "name": "Ramp", 
            "updated_at": "2015-01-29T21:33:09-08:00"
          }, 
          "ephys_stimulus_type_id": 324305749, 
          "id": 324305751, 
          "updated_at": "2015-01-29T21:33:09-08:00"
        }, 
        "ephys_stimulus_id": 324305751, 
        "ephys_sweep_tags": [], 
        "id": 396323452, 
        "leak_pa": 0.0, 
        "num_spikes": 9, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0, 
        "post_vm_mv": 0.0, 
        "pre_noise_rms_mv": 0.0401654802262783, 
        "pre_vm_mv": -73.7982177734375, 
        "slow_noise_rms_mv": 0.220393180847168, 
        "slow_vm_mv": -73.7982177734375, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 800.000010681146, 
        "stimulus_duration": 31.997495, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.0225, 
        "stimulus_units": "Amps", 
        "sweep_number": 4, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.0, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:07-08:00", 
          "description": "C1LSCOARSE141203[16]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310919, 
          "updated_at": "2015-01-29T21:58:07-08:00"
        }, 
        "ephys_stimulus_id": 324310919, 
        "ephys_sweep_tags": [], 
        "id": 396323447, 
        "leak_pa": -7.99744129180908, 
        "num_spikes": 28, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0427638366818428, 
        "post_vm_mv": -73.8689270019531, 
        "pre_noise_rms_mv": 0.0446272566914558, 
        "pre_vm_mv": -72.5650634765625, 
        "slow_noise_rms_mv": 0.176345944404602, 
        "slow_vm_mv": -72.5650634765625, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 210.00000349769, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 38, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 1.30386352539062, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:07-08:00", 
          "description": "C1LSCOARSE141203[15]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310915, 
          "updated_at": "2015-01-29T21:58:07-08:00"
        }, 
        "ephys_stimulus_id": 324310915, 
        "ephys_sweep_tags": [], 
        "id": 396323445, 
        "leak_pa": -7.99744129180908, 
        "num_spikes": 22, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0413710474967957, 
        "post_vm_mv": -72.8272399902344, 
        "pre_noise_rms_mv": 0.0437457673251629, 
        "pre_vm_mv": -71.9559326171875, 
        "slow_noise_rms_mv": 0.297846466302872, 
        "slow_vm_mv": -71.9559326171875, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 190.000001842883, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 37, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.871307373046875, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:07-08:00", 
          "description": "C1LSCOARSE141203[14]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310911, 
          "updated_at": "2015-01-29T21:58:07-08:00"
        }, 
        "ephys_stimulus_id": 324310911, 
        "ephys_sweep_tags": [], 
        "id": 396323443, 
        "leak_pa": -7.99744129180908, 
        "num_spikes": 21, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0392543375492096, 
        "post_vm_mv": -72.5036392211914, 
        "pre_noise_rms_mv": 0.0393037050962448, 
        "pre_vm_mv": -72.1030654907227, 
        "slow_noise_rms_mv": 0.281521439552307, 
        "slow_vm_mv": -72.1030654907227, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 170.000000188075, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 36, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.40057373046875, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T21:58:05-08:00", 
          "description": "C1SSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310797, 
          "updated_at": "2015-01-29T21:58:05-08:00"
        }, 
        "ephys_stimulus_id": 324310797, 
        "ephys_sweep_tags": [], 
        "id": 396323368, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0358530394732952, 
        "post_vm_mv": -71.6928253173828, 
        "pre_noise_rms_mv": 0.0396796762943268, 
        "pre_vm_mv": -73.5050048828125, 
        "slow_noise_rms_mv": 0.135289296507835, 
        "slow_vm_mv": -73.5050048828125, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 17, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 1.81217956542969, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1LSCOARSE141203[6]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310837, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310837, 
        "ephys_sweep_tags": [], 
        "id": 396323401, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.046930156648159, 
        "post_vm_mv": -71.696891784668, 
        "pre_noise_rms_mv": 0.0421544797718525, 
        "pre_vm_mv": -72.1745986938477, 
        "slow_noise_rms_mv": 0.144713699817657, 
        "slow_vm_mv": -72.1745986938477, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 9.99999996004197, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 28, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.477706909179688, 
        "workflow_state": "manual_failed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1LSCOARSE141203[7]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310841, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310841, 
        "ephys_sweep_tags": [], 
        "id": 396323404, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0400526039302349, 
        "post_vm_mv": -71.804801940918, 
        "pre_noise_rms_mv": 0.036600898951292, 
        "pre_vm_mv": -71.5464630126953, 
        "slow_noise_rms_mv": 0.205636784434319, 
        "slow_vm_mv": -71.5464630126953, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 29.9999990127642, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 29, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.258338928222656, 
        "workflow_state": "manual_failed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[3]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338123, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338123, 
        "ephys_sweep_tags": [], 
        "id": 396323600, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.035027839243412, 
        "post_vm_mv": -73.2114715576172, 
        "pre_noise_rms_mv": 0.0415959544479847, 
        "pre_vm_mv": -73.2385482788086, 
        "slow_noise_rms_mv": 0.187070429325104, 
        "slow_vm_mv": -73.2385482788086, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.040995, 
        "stimulus_interval": 0.019, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 83, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.0270767211914062, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[4]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338127, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338127, 
        "ephys_sweep_tags": [], 
        "id": 396323604, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0420295111835003, 
        "post_vm_mv": -74.1104354858398, 
        "pre_noise_rms_mv": 0.0353056378662586, 
        "pre_vm_mv": -73.7717056274414, 
        "slow_noise_rms_mv": 0.246308460831642, 
        "slow_vm_mv": -73.7717056274414, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.048995, 
        "stimulus_interval": 0.023, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 84, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.338729858398438, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 0.0, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:33:07-08:00", 
          "description": "EXTPSMOKET141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305553, 
            "name": "Test", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305553, 
          "id": 324305555, 
          "updated_at": "2015-01-29T21:33:07-08:00"
        }, 
        "ephys_stimulus_id": 324305555, 
        "ephys_sweep_tags": [], 
        "id": 396323328, 
        "leak_pa": 0.0, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0, 
        "post_vm_mv": 0.0, 
        "pre_noise_rms_mv": 0.0, 
        "pre_vm_mv": 0.0, 
        "slow_noise_rms_mv": 0.0, 
        "slow_vm_mv": 0.0, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 9.99999977648258, 
        "stimulus_duration": 0.069995, 
        "stimulus_interval": 0.05, 
        "stimulus_start_time": 0.03, 
        "stimulus_units": "Volts", 
        "sweep_number": 0, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.0, 
        "workflow_state": "unknown"
      }, 
      {
        "bridge_balance_mohm": 0.0, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:33:07-08:00", 
          "description": "EXTPINBATH141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305553, 
            "name": "Test", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305553, 
          "id": 324305559, 
          "updated_at": "2015-01-29T21:33:07-08:00"
        }, 
        "ephys_stimulus_id": 324305559, 
        "ephys_sweep_tags": [], 
        "id": 396323330, 
        "leak_pa": 0.0, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0, 
        "post_vm_mv": 0.0, 
        "pre_noise_rms_mv": 0.0, 
        "pre_vm_mv": 0.0, 
        "slow_noise_rms_mv": 0.0, 
        "slow_vm_mv": 0.0, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 9.99999977648258, 
        "stimulus_duration": 0.069995, 
        "stimulus_interval": 0.05, 
        "stimulus_start_time": 0.03, 
        "stimulus_units": "Volts", 
        "sweep_number": 1, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.0, 
        "workflow_state": "unknown"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:33:08-08:00", 
          "description": "EXTPEXPEND141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305553, 
            "name": "Test", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305553, 
          "id": 324305637, 
          "updated_at": "2015-01-29T21:33:08-08:00"
        }, 
        "ephys_stimulus_id": 324305637, 
        "ephys_sweep_tags": [], 
        "id": 396323346, 
        "leak_pa": -39.9872055053711, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0, 
        "post_vm_mv": 0.0, 
        "pre_noise_rms_mv": 0.0, 
        "pre_vm_mv": 0.0, 
        "slow_noise_rms_mv": 0.0, 
        "slow_vm_mv": 0.0, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 9.99999977648258, 
        "stimulus_duration": 0.069995, 
        "stimulus_interval": 0.05, 
        "stimulus_start_time": 0.03, 
        "stimulus_units": "Volts", 
        "sweep_number": 104, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.0, 
        "workflow_state": "unknown"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:33:08-08:00", 
          "description": "EXTPGGAEND141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305553, 
            "name": "Test", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305553, 
          "id": 324305641, 
          "updated_at": "2015-01-29T21:33:08-08:00"
        }, 
        "ephys_stimulus_id": 324305641, 
        "ephys_sweep_tags": [], 
        "id": 396323350, 
        "leak_pa": -39.9872055053711, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0, 
        "post_vm_mv": 0.0, 
        "pre_noise_rms_mv": 0.0, 
        "pre_vm_mv": 0.0, 
        "slow_noise_rms_mv": 0.0, 
        "slow_vm_mv": 0.0, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 9.99999977648258, 
        "stimulus_duration": 0.069995, 
        "stimulus_interval": 0.05, 
        "stimulus_start_time": 0.03, 
        "stimulus_units": "Volts", 
        "sweep_number": 105, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.0, 
        "workflow_state": "unknown"
      }, 
      {
        "bridge_balance_mohm": 0.0, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:33:09-08:00", 
          "description": "EXTPBREAKN141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305553, 
            "name": "Test", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305553, 
          "id": 324305705, 
          "updated_at": "2015-01-29T21:33:09-08:00"
        }, 
        "ephys_stimulus_id": 324305705, 
        "ephys_sweep_tags": [], 
        "id": 396323406, 
        "leak_pa": 0.0, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0, 
        "post_vm_mv": 0.0, 
        "pre_noise_rms_mv": 0.0, 
        "pre_vm_mv": 0.0, 
        "slow_noise_rms_mv": 0.0, 
        "slow_vm_mv": 0.0, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 9.99999977648258, 
        "stimulus_duration": 0.069995, 
        "stimulus_interval": 0.05, 
        "stimulus_start_time": 0.03, 
        "stimulus_units": "Volts", 
        "sweep_number": 3, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.0, 
        "workflow_state": "unknown"
      }, 
      {
        "bridge_balance_mohm": 0.0, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:33:08-08:00", 
          "description": "EXTPCllATT141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305553, 
            "name": "Test", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305553, 
          "id": 324305665, 
          "updated_at": "2015-01-29T21:33:08-08:00"
        }, 
        "ephys_stimulus_id": 324305665, 
        "ephys_sweep_tags": [], 
        "id": 396323378, 
        "leak_pa": 0.0, 
        "num_spikes": 0, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0, 
        "post_vm_mv": 0.0, 
        "pre_noise_rms_mv": 0.0, 
        "pre_vm_mv": 0.0, 
        "slow_noise_rms_mv": 0.0, 
        "slow_vm_mv": 0.0, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 9.99999977648258, 
        "stimulus_duration": 0.069995, 
        "stimulus_interval": 0.05, 
        "stimulus_start_time": 0.03, 
        "stimulus_units": "Volts", 
        "sweep_number": 2, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.0, 
        "workflow_state": "unknown"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[7]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338141, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338141, 
        "ephys_sweep_tags": [], 
        "id": 396323587, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0352929159998894, 
        "post_vm_mv": -73.169921875, 
        "pre_noise_rms_mv": 0.0338610708713531, 
        "pre_vm_mv": -73.5805969238281, 
        "slow_noise_rms_mv": 0.198360458016396, 
        "slow_vm_mv": -73.5805969238281, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.072995, 
        "stimulus_interval": 0.035, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 79, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.410675048828125, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:15-08:00", 
          "description": "C2SSTRIPLE141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338111, 
          "updated_at": "2015-01-29T22:53:15-08:00"
        }, 
        "ephys_stimulus_id": 324338111, 
        "ephys_sweep_tags": [], 
        "id": 396323592, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0363268032670021, 
        "post_vm_mv": -73.3690032958984, 
        "pre_noise_rms_mv": 0.0389319099485874, 
        "pre_vm_mv": -73.3859329223633, 
        "slow_noise_rms_mv": 0.242742508649826, 
        "slow_vm_mv": -73.3859329223633, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.016995, 
        "stimulus_interval": 0.007, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 80, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.0169296264648438, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[1]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338115, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338115, 
        "ephys_sweep_tags": [], 
        "id": 396323594, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0395385921001434, 
        "post_vm_mv": -73.3211059570312, 
        "pre_noise_rms_mv": 0.043050542473793, 
        "pre_vm_mv": -73.5779342651367, 
        "slow_noise_rms_mv": 0.205265551805496, 
        "slow_vm_mv": -73.5779342651367, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.024995, 
        "stimulus_interval": 0.011, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 81, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.256828308105469, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:05-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[2]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338119, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338119, 
        "ephys_sweep_tags": [], 
        "id": 396323597, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0375289209187031, 
        "post_vm_mv": -73.3284530639648, 
        "pre_noise_rms_mv": 0.0366559848189354, 
        "pre_vm_mv": -73.0539169311523, 
        "slow_noise_rms_mv": 0.326934158802032, 
        "slow_vm_mv": -73.0539169311523, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.032995, 
        "stimulus_interval": 0.015, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 82, 
        "updated_at": "2015-10-01T14:46:15-07:00", 
        "vm_delta_mv": 0.2745361328125, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1LSCOARSE141203[5]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310833, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310833, 
        "ephys_sweep_tags": [], 
        "id": 396323397, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 0, 
        "peak_deflection": -76.34375, 
        "post_noise_rms_mv": 0.0391541942954063, 
        "post_vm_mv": -72.0367660522461, 
        "pre_noise_rms_mv": 0.0414051413536072, 
        "pre_vm_mv": -71.9976196289062, 
        "slow_noise_rms_mv": 0.180227696895599, 
        "slow_vm_mv": -71.9976196289062, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": -9.99999996004197, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 27, 
        "updated_at": "2015-07-08T05:57:52-07:00", 
        "vm_delta_mv": 0.0391464233398438, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1LSCOARSE141203[8]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310847, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310847, 
        "ephys_sweep_tags": [], 
        "id": 396323408, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 0, 
        "peak_deflection": -48.09375, 
        "post_noise_rms_mv": 0.043704766780138, 
        "post_vm_mv": -71.5417327880859, 
        "pre_noise_rms_mv": 0.0406602211296558, 
        "pre_vm_mv": -71.6712112426758, 
        "slow_noise_rms_mv": 0.288515567779541, 
        "slow_vm_mv": -71.6712112426758, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 50.0000006675716, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 30, 
        "updated_at": "2015-07-08T05:57:52-07:00", 
        "vm_delta_mv": 0.129478454589844, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 0.8, 
          "created_at": "2015-01-29T22:18:33-08:00", 
          "description": "C1LSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324321357, 
          "updated_at": "2015-01-29T22:18:33-08:00"
        }, 
        "ephys_stimulus_id": 324321357, 
        "ephys_sweep_tags": [], 
        "id": 396323456, 
        "leak_pa": -12.028151512146, 
        "num_spikes": 0, 
        "peak_deflection": -44.28125, 
        "post_noise_rms_mv": 0.0415375158190727, 
        "post_vm_mv": -72.9263229370117, 
        "pre_noise_rms_mv": 0.0367179661989212, 
        "pre_vm_mv": -73.3893051147461, 
        "slow_noise_rms_mv": 0.140569120645523, 
        "slow_vm_mv": -73.3893051147461, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 79.9999996803358, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 41, 
        "updated_at": "2015-07-08T05:57:52-07:00", 
        "vm_delta_mv": 0.462982177734375, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1LSCOARSE141203[3]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310825, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310825, 
        "ephys_sweep_tags": [], 
        "id": 396323392, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 0, 
        "peak_deflection": -89.53125, 
        "post_noise_rms_mv": 0.0390310399234295, 
        "post_vm_mv": -73.2279891967773, 
        "pre_noise_rms_mv": 0.0365938991308212, 
        "pre_vm_mv": -73.35302734375, 
        "slow_noise_rms_mv": 0.156366005539894, 
        "slow_vm_mv": -73.35302734375, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": -50.0000006675716, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 25, 
        "updated_at": "2015-07-08T05:57:54-07:00", 
        "vm_delta_mv": 0.125038146972656, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 0.7, 
          "created_at": "2015-01-29T22:18:33-08:00", 
          "description": "C1LSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324321349, 
          "updated_at": "2015-01-29T22:18:33-08:00"
        }, 
        "ephys_stimulus_id": 324321349, 
        "ephys_sweep_tags": [], 
        "id": 396323454, 
        "leak_pa": -12.028151512146, 
        "num_spikes": 0, 
        "peak_deflection": -48.90625, 
        "post_noise_rms_mv": 0.0441768728196621, 
        "post_vm_mv": -73.1199493408203, 
        "pre_noise_rms_mv": 0.0386229529976845, 
        "pre_vm_mv": -72.9328842163086, 
        "slow_noise_rms_mv": 0.16550201177597, 
        "slow_vm_mv": -72.9328842163086, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 69.9999988529321, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 40, 
        "updated_at": "2015-07-08T05:57:54-07:00", 
        "vm_delta_mv": 0.187065124511719, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1LSCOARSE141203[4]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310829, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310829, 
        "ephys_sweep_tags": [], 
        "id": 396323394, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 0, 
        "peak_deflection": -83.34375, 
        "post_noise_rms_mv": 0.0405874624848366, 
        "post_vm_mv": -72.389518737793, 
        "pre_noise_rms_mv": 0.0415650308132172, 
        "pre_vm_mv": -72.2010955810547, 
        "slow_noise_rms_mv": 0.131545454263687, 
        "slow_vm_mv": -72.2010955810547, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": -29.9999990127642, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 26, 
        "updated_at": "2015-07-08T05:57:52-07:00", 
        "vm_delta_mv": 0.188423156738281, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T22:18:36-08:00", 
          "description": "C2NSRMPRHE141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305829, 
            "name": "Ramp to Rheobase", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305829, 
          "id": 324321519, 
          "updated_at": "2015-01-29T22:18:36-08:00"
        }, 
        "ephys_stimulus_id": 324321519, 
        "ephys_sweep_tags": [], 
        "id": 396323542, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 57, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0380563996732235, 
        "post_vm_mv": -75.0327987670898, 
        "pre_noise_rms_mv": 0.0397848822176456, 
        "pre_vm_mv": -73.5854339599609, 
        "slow_noise_rms_mv": 0.180791780352592, 
        "slow_vm_mv": -73.5854339599609, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 255.750004507505, 
        "stimulus_duration": 29.999975, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 2.02002, 
        "stimulus_units": "Amps", 
        "sweep_number": 63, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 1.44736480712891, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:15-08:00", 
          "description": "C2SSTRIPLE141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338111, 
          "updated_at": "2015-01-29T22:53:15-08:00"
        }, 
        "ephys_stimulus_id": 324338111, 
        "ephys_sweep_tags": [], 
        "id": 396323544, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.037385605275631, 
        "post_vm_mv": -73.5233383178711, 
        "pre_noise_rms_mv": 0.0444515310227871, 
        "pre_vm_mv": -73.5065689086914, 
        "slow_noise_rms_mv": 0.173128440976143, 
        "slow_vm_mv": -73.5065689086914, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.016995, 
        "stimulus_interval": 0.007, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 64, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.0167694091796875, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[1]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338115, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338115, 
        "ephys_sweep_tags": [], 
        "id": 396323548, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0440761931240559, 
        "post_vm_mv": -73.6457672119141, 
        "pre_noise_rms_mv": 0.0434141494333744, 
        "pre_vm_mv": -73.6492462158203, 
        "slow_noise_rms_mv": 0.180153012275696, 
        "slow_vm_mv": -73.6492462158203, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.024995, 
        "stimulus_interval": 0.011, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 65, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.00347900390625, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[2]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338119, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338119, 
        "ephys_sweep_tags": [], 
        "id": 396323552, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0393132194876671, 
        "post_vm_mv": -73.7283935546875, 
        "pre_noise_rms_mv": 0.0338524095714092, 
        "pre_vm_mv": -73.6042861938477, 
        "slow_noise_rms_mv": 0.159617558121681, 
        "slow_vm_mv": -73.6042861938477, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.032995, 
        "stimulus_interval": 0.015, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 66, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.124107360839844, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:04-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T22:53:16-08:00", 
          "description": "C2SSTRIPLE141203[3]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:10-08:00", 
            "id": 324305839, 
            "name": "Short Square - Triple", 
            "updated_at": "2015-01-29T21:33:10-08:00"
          }, 
          "ephys_stimulus_type_id": 324305839, 
          "id": 324338123, 
          "updated_at": "2015-01-29T22:53:16-08:00"
        }, 
        "ephys_stimulus_id": 324338123, 
        "ephys_sweep_tags": [], 
        "id": 396323554, 
        "leak_pa": -9.98080635070801, 
        "num_spikes": 3, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0420318059623241, 
        "post_vm_mv": -73.6357803344727, 
        "pre_noise_rms_mv": 0.0342406034469604, 
        "pre_vm_mv": -73.4720077514648, 
        "slow_noise_rms_mv": 0.228944271802902, 
        "slow_vm_mv": -73.4720077514648, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.040995, 
        "stimulus_interval": 0.019, 
        "stimulus_start_time": 2.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 67, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.163772583007812, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T21:58:05-08:00", 
          "description": "C1SSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310797, 
          "updated_at": "2015-01-29T21:58:05-08:00"
        }, 
        "ephys_stimulus_id": 324310797, 
        "ephys_sweep_tags": [], 
        "id": 396323382, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0378184206783772, 
        "post_vm_mv": -70.87744140625, 
        "pre_noise_rms_mv": 0.0403490774333477, 
        "pre_vm_mv": -72.5947113037109, 
        "slow_noise_rms_mv": 0.153314620256424, 
        "slow_vm_mv": -72.5947113037109, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 21, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 1.71726989746094, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 6.0, 
          "created_at": "2015-01-29T21:58:05-08:00", 
          "description": "C1SSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:07-08:00", 
            "id": 324305563, 
            "name": "Short Square", 
            "updated_at": "2015-01-29T21:33:07-08:00"
          }, 
          "ephys_stimulus_type_id": 324305563, 
          "id": 324310797, 
          "updated_at": "2015-01-29T21:58:05-08:00"
        }, 
        "ephys_stimulus_id": 324310797, 
        "ephys_sweep_tags": [], 
        "id": 396323366, 
        "leak_pa": -4.990403175354, 
        "num_spikes": 1, 
        "peak_deflection": null, 
        "post_noise_rms_mv": 0.0432348102331161, 
        "post_vm_mv": -71.8769836425781, 
        "pre_noise_rms_mv": 0.0401340462267399, 
        "pre_vm_mv": -73.5547409057617, 
        "slow_noise_rms_mv": 0.11938589066267, 
        "slow_vm_mv": -73.5547409057617, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 599.999994133071, 
        "stimulus_duration": 0.002995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 16, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 1.67775726318359, 
        "workflow_state": "manual_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1LSCOARSE141203[2]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310821, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310821, 
        "ephys_sweep_tags": [], 
        "id": 396323390, 
        "leak_pa": -6.97376823425293, 
        "num_spikes": 0, 
        "peak_deflection": -93.40625, 
        "post_noise_rms_mv": 0.0433307066559792, 
        "post_vm_mv": -73.8148193359375, 
        "pre_noise_rms_mv": 0.0394219271838665, 
        "pre_vm_mv": -73.3753662109375, 
        "slow_noise_rms_mv": 0.185792118310928, 
        "slow_vm_mv": -73.3753662109375, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": -69.9999988529321, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 24, 
        "updated_at": "2015-07-08T05:57:54-07:00", 
        "vm_delta_mv": 0.439453125, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:03-08:00", 
        "ephys_stimulus": {
          "amplitude": 0.6, 
          "created_at": "2015-01-29T22:13:04-08:00", 
          "description": "C1LSFINEST141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324318916, 
          "updated_at": "2015-01-29T22:13:04-08:00"
        }, 
        "ephys_stimulus_id": 324318916, 
        "ephys_sweep_tags": [], 
        "id": 396323449, 
        "leak_pa": -7.99744129180908, 
        "num_spikes": 0, 
        "peak_deflection": -51.7187538146973, 
        "post_noise_rms_mv": 0.0381619110703468, 
        "post_vm_mv": -72.2404022216797, 
        "pre_noise_rms_mv": 0.0412812754511833, 
        "pre_vm_mv": -72.1742477416992, 
        "slow_noise_rms_mv": 0.184022128582001, 
        "slow_vm_mv": -72.1742477416992, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": 59.9999980255284, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 39, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.0661544799804688, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1LSCOARSE141203[1]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310817, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310817, 
        "ephys_sweep_tags": [], 
        "id": 396323388, 
        "leak_pa": -6.97376823425293, 
        "num_spikes": 0, 
        "peak_deflection": -96.9687576293945, 
        "post_noise_rms_mv": 0.0375870503485203, 
        "post_vm_mv": -73.6441040039062, 
        "pre_noise_rms_mv": 0.0391277000308037, 
        "pre_vm_mv": -73.8342666625977, 
        "slow_noise_rms_mv": 0.149396568536758, 
        "slow_vm_mv": -73.8342666625977, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": -90.0000005077395, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 23, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.190162658691406, 
        "workflow_state": "auto_passed"
      }, 
      {
        "bridge_balance_mohm": 14.224663734436, 
        "created_at": "2015-03-02T15:04:02-08:00", 
        "ephys_stimulus": {
          "amplitude": 1.0, 
          "created_at": "2015-01-29T21:58:06-08:00", 
          "description": "C1LSCOARSE141203[0]", 
          "ephys_stimulus_type": {
            "created_at": "2015-01-29T21:33:08-08:00", 
            "id": 324305675, 
            "name": "Long Square", 
            "updated_at": "2015-01-29T21:33:08-08:00"
          }, 
          "ephys_stimulus_type_id": 324305675, 
          "id": 324310813, 
          "updated_at": "2015-01-29T21:58:06-08:00"
        }, 
        "ephys_stimulus_id": 324310813, 
        "ephys_sweep_tags": [], 
        "id": 396323384, 
        "leak_pa": -6.97376823425293, 
        "num_spikes": 0, 
        "peak_deflection": -100.156257629395, 
        "post_noise_rms_mv": 0.0376262404024601, 
        "post_vm_mv": -74.3083114624023, 
        "pre_noise_rms_mv": 0.0403164774179459, 
        "pre_vm_mv": -73.7102813720703, 
        "slow_noise_rms_mv": 0.212610512971878, 
        "slow_vm_mv": -73.7102813720703, 
        "specimen_id": 318733871, 
        "stimulus_amplitude": -110.000002162547, 
        "stimulus_duration": 0.999995, 
        "stimulus_interval": 0.0, 
        "stimulus_start_time": 1.02, 
        "stimulus_units": "Amps", 
        "sweep_number": 22, 
        "updated_at": "2015-10-01T14:46:13-07:00", 
        "vm_delta_mv": 0.598030090332031, 
        "workflow_state": "auto_passed"
      }
    ], 
    "external_specimen_name": null, 
    "facs_well_id": null, 
    "frozen_at": null, 
    "hemisphere_id": 2, 
    "histology_well_name": null, 
    "id": 318733871, 
    "location_id": null, 
    "name": "Nr5a1-Cre;Ai14-169248.04.02.01", 
    "neuron_reconstructions": [
      {
        "average_bifurcation_angle_local": 82.2651, 
        "average_bifurcation_angle_remote": 80.9024, 
        "average_contraction": 0.891136, 
        "average_diameter": 0.424899, 
        "average_fragmentation": 38.9318, 
        "average_parent_daughter_ratio": 0.828312, 
        "created_at": "2015-02-06T11:08:53-08:00", 
        "hausdorff_dimension": 1.17512, 
        "id": 326737179, 
        "manual": true, 
        "max_branch_order": 8, 
        "max_euclidean_distance": 411.175, 
        "max_path_distance": 482.472, 
        "nodes_over_branches": 39.955, 
        "number_bifurcations": 20, 
        "number_branches": 44, 
        "number_nodes": 1758, 
        "number_stems": 6, 
        "number_tips": 25, 
        "overall_depth": 103.41, 
        "overall_height": 463.378, 
        "overall_width": 246.074, 
        "scale_factor_x": 0.1144, 
        "scale_factor_y": 0.1144, 
        "scale_factor_z": 0.28, 
        "soma_surface": 610.486, 
        "specimen_id": 318733871, 
        "superseded": true, 
        "total_length": 2159.88, 
        "total_surface": 2869.99, 
        "total_volume": 363.123, 
        "updated_at": "2015-02-12T10:50:05-08:00", 
        "user_id": 250, 
        "well_known_files": [
          {
            "attachable_id": 326737179, 
            "attachable_type": "NeuronReconstruction", 
            "content_type": "text/plain; charset=us-ascii", 
            "created_at": "2015-02-06T11:08:54-08:00", 
            "file_source_id": null, 
            "filename": "Nr5a1-Cre_Ai14_IVSCC_-169248.04.02.01_326737179_m.swc", 
            "id": 326737181, 
            "published_at": null, 
            "size": 74992, 
            "storage_directory": "/projects/mousecelltypes/vol1/prod155/specimen_318733871/", 
            "updated_at": "2015-02-06T11:08:54-08:00", 
            "well_known_file_type": {
              "created_at": "2014-05-16T12:29:39-07:00", 
              "id": 303941301, 
              "name": "3DNeuronReconstruction", 
              "updated_at": "2014-05-16T12:29:39-07:00"
            }, 
            "well_known_file_type_id": 303941301, 
            "workflow_state": null
          }
        ]
      }, 
      {
        "average_bifurcation_angle_local": 85.3917, 
        "average_bifurcation_angle_remote": 98.8257, 
        "average_contraction": 0.73481, 
        "average_diameter": 0.359785, 
        "average_fragmentation": 18.9057, 
        "average_parent_daughter_ratio": 1.06767, 
        "created_at": "2015-01-29T20:23:08-08:00", 
        "hausdorff_dimension": 1.21225, 
        "id": 324291233, 
        "manual": false, 
        "max_branch_order": 9, 
        "max_euclidean_distance": 176.473, 
        "max_path_distance": 194.904, 
        "nodes_over_branches": 19.9245283018868, 
        "number_bifurcations": 25, 
        "number_branches": 53, 
        "number_nodes": 1056, 
        "number_stems": 5, 
        "number_tips": 29, 
        "overall_depth": 40.2531, 
        "overall_height": 153.476, 
        "overall_width": 206.151, 
        "scale_factor_x": 0.1144, 
        "scale_factor_y": 0.1144, 
        "scale_factor_z": 0.28, 
        "soma_surface": 332.412, 
        "specimen_id": 318733871, 
        "superseded": true, 
        "total_length": 1363.13, 
        "total_surface": 1542.32, 
        "total_volume": 168.265, 
        "updated_at": "2015-02-06T11:08:48-08:00", 
        "user_id": null, 
        "well_known_files": [
          {
            "attachable_id": 324291233, 
            "attachable_type": "NeuronReconstruction", 
            "content_type": "text/plain; charset=us-ascii", 
            "created_at": "2015-01-29T20:23:09-08:00", 
            "file_source_id": null, 
            "filename": "Nr5a1-Cre_Ai14_IVSCC_-169248.04.02.01_324291233_m.swc", 
            "id": 324291238, 
            "published_at": null, 
            "size": 44867, 
            "storage_directory": "/projects/mousecelltypes/vol1/prod155/specimen_318733871/", 
            "updated_at": "2015-01-29T20:23:09-08:00", 
            "well_known_file_type": {
              "created_at": "2014-05-16T12:29:39-07:00", 
              "id": 303941301, 
              "name": "3DNeuronReconstruction", 
              "updated_at": "2014-05-16T12:29:39-07:00"
            }, 
            "well_known_file_type_id": 303941301, 
            "workflow_state": null
          }
        ]
      }, 
      {
        "average_bifurcation_angle_local": 0.0, 
        "average_bifurcation_angle_remote": 0.0, 
        "average_contraction": 0.993603, 
        "average_diameter": 0.394682, 
        "average_fragmentation": 1.0, 
        "average_parent_daughter_ratio": 0.122453, 
        "created_at": "2015-02-06T11:08:48-08:00", 
        "hausdorff_dimension": 1.14685, 
        "id": 326737168, 
        "manual": true, 
        "max_branch_order": 1, 
        "max_euclidean_distance": 173.051, 
        "max_path_distance": 7.13842, 
        "nodes_over_branches": 508.0, 
        "number_bifurcations": 0, 
        "number_branches": 1, 
        "number_nodes": 508, 
        "number_stems": 1, 
        "number_tips": 19, 
        "overall_depth": 40.32, 
        "overall_height": 187.387, 
        "overall_width": 136.524, 
        "scale_factor_x": 0.1144, 
        "scale_factor_y": 0.1144, 
        "scale_factor_z": 0.28, 
        "soma_surface": 610.486, 
        "specimen_id": 318733871, 
        "superseded": true, 
        "total_length": 649.368, 
        "total_surface": 839.568, 
        "total_volume": 109.605, 
        "updated_at": "2015-02-06T11:17:55-08:00", 
        "user_id": 250, 
        "well_known_files": [
          {
            "attachable_id": 326737168, 
            "attachable_type": "NeuronReconstruction", 
            "content_type": "text/plain; charset=us-ascii", 
            "created_at": "2015-02-06T11:08:48-08:00", 
            "file_source_id": null, 
            "filename": "Nr5a1-Cre_Ai14_IVSCC_-169248.04.02.01_326737168_m.swc", 
            "id": 326737170, 
            "published_at": null, 
            "size": 21450, 
            "storage_directory": "/projects/mousecelltypes/vol1/prod155/specimen_318733871/", 
            "updated_at": "2015-02-06T11:08:48-08:00", 
            "well_known_file_type": {
              "created_at": "2014-05-16T12:29:39-07:00", 
              "id": 303941301, 
              "name": "3DNeuronReconstruction", 
              "updated_at": "2014-05-16T12:29:39-07:00"
            }, 
            "well_known_file_type_id": 303941301, 
            "workflow_state": null
          }
        ]
      }, 
      {
        "average_bifurcation_angle_local": 85.3913, 
        "average_bifurcation_angle_remote": 98.8256, 
        "average_contraction": 0.73481, 
        "average_diameter": 0.359784, 
        "average_fragmentation": 18.9057, 
        "average_parent_daughter_ratio": 1.06765, 
        "created_at": "2015-02-12T10:50:05-08:00", 
        "hausdorff_dimension": 1.21225, 
        "id": 328874691, 
        "manual": true, 
        "max_branch_order": 9, 
        "max_euclidean_distance": 176.473, 
        "max_path_distance": 194.904, 
        "nodes_over_branches": 19.9245283018868, 
        "number_bifurcations": 25, 
        "number_branches": 53, 
        "number_nodes": 1056, 
        "number_stems": 5, 
        "number_tips": 29, 
        "overall_depth": 40.2531, 
        "overall_height": 153.476, 
        "overall_width": 206.151, 
        "scale_factor_x": 0.1144, 
        "scale_factor_y": 0.1144, 
        "scale_factor_z": 0.28, 
        "soma_surface": 332.412, 
        "specimen_id": 318733871, 
        "superseded": true, 
        "total_length": 1363.12, 
        "total_surface": 1542.32, 
        "total_volume": 168.265, 
        "updated_at": "2015-02-12T11:10:10-08:00", 
        "user_id": 250, 
        "well_known_files": [
          {
            "attachable_id": 328874691, 
            "attachable_type": "NeuronReconstruction", 
            "content_type": "text/plain; charset=us-ascii", 
            "created_at": "2015-02-12T10:50:06-08:00", 
            "file_source_id": null, 
            "filename": "Nr5a1-Cre_Ai14_IVSCC_-169248.04.02.01_328874691_m.swc", 
            "id": 328874693, 
            "published_at": null, 
            "size": 44831, 
            "storage_directory": "/projects/mousecelltypes/vol1/prod155/specimen_318733871/", 
            "updated_at": "2015-02-12T10:50:06-08:00", 
            "well_known_file_type": {
              "created_at": "2014-05-16T12:29:39-07:00", 
              "id": 303941301, 
              "name": "3DNeuronReconstruction", 
              "updated_at": "2014-05-16T12:29:39-07:00"
            }, 
            "well_known_file_type_id": 303941301, 
            "workflow_state": null
          }
        ]
      }, 
      {
        "average_bifurcation_angle_local": 72.8762, 
        "average_bifurcation_angle_remote": 79.2273, 
        "average_contraction": 0.869187, 
        "average_diameter": 0.383292, 
        "average_fragmentation": 40.4, 
        "average_parent_daughter_ratio": 0.995908, 
        "created_at": "2015-02-12T10:50:17-08:00", 
        "hausdorff_dimension": 1.13073, 
        "id": 328874724, 
        "manual": true, 
        "max_branch_order": 6, 
        "max_euclidean_distance": 267.871, 
        "max_path_distance": 285.128, 
        "nodes_over_branches": 41.425, 
        "number_bifurcations": 17, 
        "number_branches": 40, 
        "number_nodes": 1657, 
        "number_stems": 8, 
        "number_tips": 24, 
        "overall_depth": 60.5338, 
        "overall_height": 280.407, 
        "overall_width": 307.497, 
        "scale_factor_x": 0.1144, 
        "scale_factor_y": 0.1144, 
        "scale_factor_z": 0.28, 
        "soma_surface": 332.412, 
        "specimen_id": 318733871, 
        "superseded": true, 
        "total_length": 1983.17, 
        "total_surface": 2399.98, 
        "total_volume": 270.642, 
        "updated_at": "2015-03-09T16:36:48-07:00", 
        "user_id": 250, 
        "well_known_files": [
          {
            "attachable_id": 328874724, 
            "attachable_type": "NeuronReconstruction", 
            "content_type": "text/plain; charset=us-ascii", 
            "created_at": "2015-02-12T10:50:17-08:00", 
            "file_source_id": null, 
            "filename": "Nr5a1-Cre_Ai14_IVSCC_-169248.04.02.01_328874724_m.swc", 
            "id": 328874726, 
            "published_at": null, 
            "size": 70938, 
            "storage_directory": "/projects/mousecelltypes/vol1/prod155/specimen_318733871/", 
            "updated_at": "2015-02-12T10:50:17-08:00", 
            "well_known_file_type": {
              "created_at": "2014-05-16T12:29:39-07:00", 
              "id": 303941301, 
              "name": "3DNeuronReconstruction", 
              "updated_at": "2014-05-16T12:29:39-07:00"
            }, 
            "well_known_file_type_id": 303941301, 
            "workflow_state": null
          }
        ]
      }, 
      {
        "average_bifurcation_angle_local": 72.9421, 
        "average_bifurcation_angle_remote": 79.422, 
        "average_contraction": 0.865652, 
        "average_diameter": 0.386189, 
        "average_fragmentation": 40.9474, 
        "average_parent_daughter_ratio": 1.01323, 
        "created_at": "2015-03-09T16:36:48-07:00", 
        "hausdorff_dimension": 1.12508, 
        "id": 403165543, 
        "manual": true, 
        "max_branch_order": 6, 
        "max_euclidean_distance": 267.871, 
        "max_path_distance": 285.128, 
        "nodes_over_branches": 41.974, 
        "number_bifurcations": 16, 
        "number_branches": 38, 
        "number_nodes": 1595, 
        "number_stems": 8, 
        "number_tips": 23, 
        "overall_depth": 60.5338, 
        "overall_height": 280.407, 
        "overall_width": 307.497, 
        "scale_factor_x": 0.1144, 
        "scale_factor_y": 0.1144, 
        "scale_factor_z": 0.28, 
        "soma_surface": 332.412, 
        "specimen_id": 318733871, 
        "superseded": true, 
        "total_length": 1911.04, 
        "total_surface": 2330.19, 
        "total_volume": 264.918, 
        "updated_at": "2015-11-20T16:57:05-08:00", 
        "user_id": 250, 
        "well_known_files": [
          {
            "attachable_id": 403165543, 
            "attachable_type": "NeuronReconstruction", 
            "content_type": "text/plain; charset=us-ascii", 
            "created_at": "2015-03-09T16:36:48-07:00", 
            "file_source_id": null, 
            "filename": "Nr5a1-Cre_Ai14_IVSCC_-169248.04.02.01_403165543_m.swc", 
            "id": 403165565, 
            "published_at": null, 
            "size": 68212, 
            "storage_directory": "/projects/mousecelltypes/vol1/prod155/specimen_318733871/", 
            "updated_at": "2015-03-09T16:36:48-07:00", 
            "well_known_file_type": {
              "created_at": "2014-05-16T12:29:39-07:00", 
              "id": 303941301, 
              "name": "3DNeuronReconstruction", 
              "updated_at": "2014-05-16T12:29:39-07:00"
            }, 
            "well_known_file_type_id": 303941301, 
            "workflow_state": null
          }
        ]
      }, 
      {
        "average_bifurcation_angle_local": 70.8819434043627, 
        "average_bifurcation_angle_remote": 78.2563982076947, 
        "average_contraction": 0.880835021187539, 
        "average_diameter": 0.311525009419645, 
        "average_fragmentation": 43.9428571428571, 
        "average_parent_daughter_ratio": 0.698053892074704, 
        "created_at": "2015-11-20T16:57:05-08:00", 
        "hausdorff_dimension": null, 
        "id": 491253825, 
        "manual": true, 
        "max_branch_order": 6, 
        "max_euclidean_distance": 212.511020927765, 
        "max_path_distance": 220.853847497628, 
        "nodes_over_branches": 44.9714285714286, 
        "number_bifurcations": 15, 
        "number_branches": 35, 
        "number_nodes": 1574, 
        "number_stems": 7, 
        "number_tips": 21, 
        "overall_depth": 273.94130015488, 
        "overall_height": 173.135753853, 
        "overall_width": 59.117962518888, 
        "scale_factor_x": 0.1144, 
        "scale_factor_y": 0.1144, 
        "scale_factor_z": 0.28, 
        "soma_surface": 232.051781050739, 
        "specimen_id": 318733871, 
        "superseded": false, 
        "total_length": 1434.15149446666, 
        "total_surface": 1398.00481682895, 
        "total_volume": 126.335576025326, 
        "updated_at": "2016-02-03T16:39:02-08:00", 
        "user_id": 250, 
        "well_known_files": [
          {
            "attachable_id": 491253825, 
            "attachable_type": "NeuronReconstruction", 
            "content_type": "text/plain; charset=us-ascii", 
            "created_at": "2015-11-20T16:57:05-08:00", 
            "file_source_id": null, 
            "filename": "Nr5a1-Cre_Ai14-169248.04.02.01_491253825_m.swc", 
            "id": 491253827, 
            "published_at": "2015-05-14", 
            "size": 68572, 
            "storage_directory": "/projects/mousecelltypes/vol1/prod155/specimen_318733871/", 
            "updated_at": "2015-11-20T16:57:05-08:00", 
            "well_known_file_type": {
              "created_at": "2014-05-16T12:29:39-07:00", 
              "id": 303941301, 
              "name": "3DNeuronReconstruction", 
              "updated_at": "2014-05-16T12:29:39-07:00"
            }, 
            "well_known_file_type_id": 303941301, 
            "workflow_state": null
          }, 
          {
            "attachable_id": 491253825, 
            "attachable_type": "NeuronReconstruction", 
            "content_type": "text/plain; charset=us-ascii", 
            "created_at": "2016-01-18T18:49:54-08:00", 
            "file_source_id": null, 
            "filename": "Nr5a1-Cre_Ai14-169248.04.02.01_491253825_marker_m.swc", 
            "id": 496607168, 
            "published_at": "2015-05-14", 
            "size": 600, 
            "storage_directory": "/projects/mousecelltypes/vol1/prod155/specimen_318733871/", 
            "updated_at": "2016-01-18T18:49:54-08:00", 
            "well_known_file_type": {
              "created_at": "2015-09-16T15:37:33-07:00", 
              "id": 486753749, 
              "name": "3DNeuronMarker", 
              "updated_at": "2015-09-16T15:37:33-07:00"
            }, 
            "well_known_file_type_id": 486753749, 
            "workflow_state": null
          }
        ]
      }
    ], 
    "normalization_group_id": null, 
    "parent_id": 318694179, 
    "parent_x_coord": 0, 
    "parent_y_coord": 0, 
    "parent_z_coord": 1, 
    "plane_of_section_id": 1, 
    "postmortem_interval_id": null, 
    "preparation_method_id": null, 
    "priority": null, 
    "project": {
      "code": "T301", 
      "created_at": "2014-07-01T09:52:32-07:00", 
      "current_directory": 659, 
      "failed_trigger_dir": "/projects/incoming/mousecelltypes/failed_trigger/", 
      "file_storage": "/data/aibstemp/josem/WellKnownFiles/", 
      "id": 305094322, 
      "incoming_directory": null, 
      "name": "in vitro Single Cell Characterization", 
      "non_cached_schema_name": null, 
      "process_triggers": true, 
      "subdirectory_count": 337, 
      "trigger_dir": "/projects/incoming/mousecelltypes/trigger/", 
      "updated_at": "2016-03-21T11:17:41-07:00"
    }, 
    "project_id": 305094322, 
    "reference_space_id": 9, 
    "rna_integrity_number": null, 
    "specimen_preparation_method_id": null, 
    "specimen_set_id": null, 
    "specimen_tags": [
      {
        "ar_association_key_name": "318733871", 
        "created_at": "2015-03-24T12:13:03-07:00", 
        "description": "neurons with no truncation", 
        "id": 470927414, 
        "is_public": true, 
        "name": "apical - intact", 
        "updated_at": "2015-03-24T12:13:03-07:00"
      }, 
      {
        "ar_association_key_name": "318733871", 
        "created_at": "2015-03-24T12:15:22-07:00", 
        "description": "initial high-level dendrite type to support May release search - spiny", 
        "id": 470928297, 
        "is_public": true, 
        "name": "dendrite type - spiny", 
        "updated_at": "2015-03-24T12:15:22-07:00"
      }
    ], 
    "storage_directory": "/projects/mousecelltypes/vol1/prod155/specimen_318733871/", 
    "structure_id": 721, 
    "task_flow_id": null, 
    "tissue_ph": null, 
    "tissue_processing_id": null, 
    "updated_at": "2015-12-10T19:52:02-08:00", 
    "updated_by": null
  }, 
  "specimen_id": 318733871, 
  "storage_directory": "/projects/mousecelltypes/vol1/prod192/neuronal_model_329322394/", 
  "updated_at": "2016-01-21T23:24:14-08:00", 
  "well_known_files": [
    {
      "attachable_id": 329322394, 
      "attachable_type": "NeuronalModel", 
      "content_type": "application/json", 
      "created_at": "2016-01-21T23:24:14-08:00", 
      "file_source_id": null, 
      "filename": "318733871_fit.json", 
      "id": 497237577, 
      "published_at": "2015-05-14", 
      "size": null, 
      "storage_directory": "/projects/mousecelltypes/vol1/prod192/neuronal_model_329322394/", 
      "updated_at": "2016-01-21T23:24:14-08:00", 
      "well_known_file_type": {
        "created_at": "2015-02-13T11:41:41-08:00", 
        "id": 329230374, 
        "name": "NeuronalModelParameters", 
        "updated_at": "2015-02-13T11:41:41-08:00"
      }, 
      "well_known_file_type_id": 329230374, 
      "workflow_state": null
    }
  ], 
  "workflow_state": "has_been_fit"
}
'''


def mock_import(mod, *args):
    if mod == "neuron":
        return MagicMock()
    return real_import(mod, *args)


def mock_read_lims_file(self, lims_path):
    self.lims_path = lims_path
    self.read_json_string(LIMS_MESSAGE)
    self.lims_update_data = dict(self.lims_data)


@pytest.fixture
def run_optimize():
    rs = RunOptimize('manifest_sdk.json', 'out.json')
    
    mock.patch.object(OptimizeConfigReader,
                      'read_lims_file',
                      mock_read_lims_file)
    
    return rs


def xtest_init(run_optimize):
    assert run_optimize.input_json == 'manifest_sdk.json'
    assert run_optimize.output_json == 'out.json'
    assert run_optimize.app_config == None
    assert run_optimize.manifest == None


orig_open = open

def open_configs(n, *args):
    (_, fn) = os.path.split(n)

    if fn == 'manifest_sdk.json':
        data_string = MANIFEST_JSON
    elif fn == 'lims_message_optimize.json':
        data_string = LIMS_MESSAGE
    else:
        return orig_open(n, *args)

    return mock_open(read_data=data_string)(n, *args)


@pytest.mark.requires_neuron
@patch("os.path.exists", return_value=True)
@patch("shutil.copy")
@patch("allensdk.model.biophysical.runner.save_nwb")
@patch.object(HocUtils, "__init__")
@patch(builtins.__name__+".__import__", side_effect=mock_import)
@patch("allensdk.core.json_utilities.write")
@patch("allensdk.internal.model.biophysical.fit_stage_2.run_stage_2")
@patch("allensdk.internal.model.biophysical.fit_stage_2.prepare_stage_2")
@patch("allensdk.internal.model.biophysical.fit_stage_1.run_stage_1")
@patch("allensdk.internal.model.biophysical.fit_stage_1.prepare_stage_1")
@patch("allensdk.internal.model.biophysical.run_passive_fit.run_passive_fit")
@patch("allensdk.core.nwb_data_set.NwbDataSet")
def test_start_specimen(nwb_data_set,
                        passive_fit,
                        prepare_stage_1,
                        run_stage_1,
                        prepare_stage_2,
                        run_stage_2,
                        json_utilities_write,
                        import_mock,
                        hoc_init,
                        save_nwb,
                        shutil_copy,
                        path_exists,
                        run_optimize):
    with patch(builtins.__name__+".open", open_configs):
        fit_description = Config().load('manifest_sdk.json')
        Utils.description = fit_description
        run_optimize.start_specimen()
        
        assert True
